  
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     
C     Hessian File
C     
C     Generated by KPP-2.1 symbolic chemistry Kinetics PreProcessor
C     (http://www.cs.vt.edu/~asandu/Software/KPP)
C     KPP is distributed under GPL, the general public licence
C     (http://www.gnu.org/copyleft/gpl.html)
C     (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
C     (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
C     With important contributions from:
C     M. Damian, Villanova University, USA
C     R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
C     
C     File                 : KPP_Hessian.f90
C     Time                 : Tue Apr 18 15:57:11 2006
C     Working directory    : /home/amir/Desktop/KPP/kpp-2.1
C     Equation file        : CMAQ_CB4.kpp
C     Output root filename : CMAQ_CB4
C     
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



      MODULE KPP_HESSIAN

      USE KPP_PARS
      USE KPP_HESSIANSP

      IMPLICIT NONE

      CONTAINS


C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     
C     Hessian - function for Hessian (Jac derivative w.r.t. variables)
C     Arguments :
C     V         - Concentrations of variable species (local)
C     F         - Concentrations of fixed species (local)
C     RCT       - Rate constants (local)
C     HESS      - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
C     
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      SUBROUTINE Hessian ( V, RCT, HESS )

C     V - Concentrations of variable species (local)
      REAL(kind=dp) :: V(KPPNSP)
C     RCT - Rate constants (local)
      REAL(kind=dp) :: RCT(KPPNRCT)
C     HESS - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
      REAL(kind=dp) :: HESS(NHESS)

C     --------------------------------------------------------
C     Note: HESS is represented in coordinate sparse format:
C     HESS(m) = d^2 f_i / dv_j dv_k = d Jac_{i,j} / dv_k
C     where i = IHESS_I(m), j = IHESS_J(m), k = IHESS_K(m).
C     --------------------------------------------------------
C     Note: d^2 f_i / dv_j dv_k = d^2 f_i / dv_k dv_j,
C     therefore only the terms d^2 f_i / dv_j dv_k
C     with j <= k are computed and stored in HESS.
C     --------------------------------------------------------

C     Local variables
C     D2A - Second derivatives of equation rates
      REAL(kind=dp) :: D2A(68)

C     Computation of the second derivatives of equation rates
C     D2A(1) = d^2 A(3) / dV(34)dV(36)
      D2A(1) = RCT(3)
C     D2A(2) = d^2 A(4) / dV(32)dV(35)
      D2A(2) = RCT(4)
C     D2A(3) = d^2 A(5) / dV(32)dV(35)
      D2A(3) = RCT(5)
C     D2A(4) = d^2 A(6) / dV(32)dV(36)
      D2A(4) = RCT(6)
C     D2A(5) = d^2 A(7) / dV(34)dV(35)
      D2A(5) = RCT(7)
C     D2A(6) = d^2 A(13) / dV(31)dV(34)
      D2A(6) = RCT(13)
C     D2A(7) = d^2 A(14) / dV(30)dV(34)
      D2A(7) = RCT(14)
C     D2A(8) = d^2 A(16) / dV(33)dV(36)
      D2A(8) = RCT(16)
C     D2A(9) = d^2 A(17) / dV(33)dV(35)
      D2A(9) = RCT(17)
C     D2A(10) = d^2 A(18) / dV(33)dV(35)
      D2A(10) = RCT(18)
C     D2A(11) = d^2 A(21)/{dV(36)dV(36)}
      D2A(11) = RCT(21)*2
C     D2A(12) = d^2 A(22) / dV(35)dV(36)
      D2A(12) = RCT(22)
C     D2A(13) = d^2 A(23) / dV(31)dV(36)
      D2A(13) = RCT(23)
C     D2A(14) = d^2 A(25) / dV(11)dV(31)
      D2A(14) = RCT(25)
C     D2A(15) = d^2 A(26)/{dV(11)dV(11)}
      D2A(15) = RCT(26)*2
C     D2A(16) = d^2 A(27) / dV(31)dV(35)
      D2A(16) = RCT(27)
C     D2A(17) = d^2 A(28) / dV(14)dV(31)
      D2A(17) = RCT(28)
C     D2A(18) = d^2 A(29) / dV(30)dV(36)
      D2A(18) = RCT(29)
C     D2A(19) = d^2 A(30) / dV(30)dV(35)
      D2A(19) = RCT(30)
C     D2A(20) = d^2 A(32) / dV(12)dV(31)
      D2A(20) = RCT(32)
C     D2A(21) = d^2 A(33)/{dV(30)dV(30)}
      D2A(21) = RCT(33)*2
C     D2A(22) = d^2 A(34)/{dV(30)dV(30)}
      D2A(22) = RCT(34)*2
C     D2A(23) = d^2 A(36) / dV(5)dV(31)
      D2A(23) = RCT(36)
C     D2A(24) = d^2 A(37) / dV(18)dV(31)
      D2A(24) = RCT(37)
C     D2A(25) = d^2 A(38) / dV(25)dV(31)
      D2A(25) = RCT(38)
C     D2A(26) = d^2 A(41) / dV(25)dV(32)
      D2A(26) = RCT(41)
C     D2A(27) = d^2 A(42) / dV(25)dV(33)
      D2A(27) = RCT(42)
C     D2A(28) = d^2 A(43) / dV(28)dV(32)
      D2A(28) = RCT(43)
C     D2A(29) = d^2 A(44) / dV(28)dV(31)
      D2A(29) = RCT(44)
C     D2A(30) = d^2 A(45) / dV(28)dV(33)
      D2A(30) = RCT(45)
C     D2A(31) = d^2 A(47) / dV(29)dV(36)
      D2A(31) = RCT(47)
C     D2A(32) = d^2 A(48) / dV(29)dV(35)
      D2A(32) = RCT(48)
C     D2A(33) = d^2 A(50)/{dV(29)dV(29)}
      D2A(33) = RCT(50)*2
C     D2A(34) = d^2 A(51) / dV(29)dV(30)
      D2A(34) = RCT(51)
C     D2A(35) = d^2 A(53) / dV(24)dV(31)
      D2A(35) = RCT(53)
C     D2A(36) = d^2 A(56) / dV(15)dV(35)
      D2A(36) = RCT(56)
C     D2A(37) = d^2 A(57) / dV(23)dV(32)
      D2A(37) = RCT(57)
C     D2A(38) = d^2 A(58) / dV(23)dV(31)
      D2A(38) = RCT(58)
C     D2A(39) = d^2 A(59) / dV(23)dV(34)
      D2A(39) = RCT(59)
C     D2A(40) = d^2 A(60) / dV(23)dV(33)
      D2A(40) = RCT(60)
C     D2A(41) = d^2 A(61) / dV(19)dV(32)
      D2A(41) = RCT(61)
C     D2A(42) = d^2 A(62) / dV(19)dV(31)
      D2A(42) = RCT(62)
C     D2A(43) = d^2 A(63) / dV(19)dV(34)
      D2A(43) = RCT(63)
C     D2A(44) = d^2 A(64) / dV(8)dV(31)
      D2A(44) = RCT(64)
C     D2A(45) = d^2 A(65) / dV(13)dV(36)
      D2A(45) = RCT(65)
C     D2A(46) = d^2 A(67) / dV(16)dV(31)
      D2A(46) = RCT(67)
C     D2A(47) = d^2 A(68) / dV(16)dV(33)
      D2A(47) = RCT(68)
C     D2A(48) = d^2 A(69) / dV(7)dV(35)
      D2A(48) = RCT(69)
C     D2A(49) = d^2 A(70) / dV(10)dV(31)
      D2A(49) = RCT(70)
C     D2A(50) = d^2 A(71) / dV(20)dV(31)
      D2A(50) = RCT(71)
C     D2A(51) = d^2 A(73) / dV(20)dV(34)
      D2A(51) = RCT(73)
C     D2A(52) = d^2 A(74) / dV(17)dV(31)
      D2A(52) = RCT(74)
C     D2A(53) = d^2 A(76) / dV(26)dV(32)
      D2A(53) = RCT(76)
C     D2A(54) = d^2 A(77) / dV(26)dV(31)
      D2A(54) = RCT(77)
C     D2A(55) = d^2 A(78) / dV(26)dV(34)
      D2A(55) = RCT(78)
C     D2A(56) = d^2 A(79) / dV(26)dV(33)
      D2A(56) = RCT(79)
C     D2A(57) = d^2 A(80) / dV(22)dV(36)
      D2A(57) = RCT(80)
C     D2A(58) = d^2 A(81)/{dV(22)dV(22)}
      D2A(58) = RCT(81)*2
C     D2A(59) = d^2 A(82) / dV(21)dV(36)
      D2A(59) = RCT(82)
C     D2A(60) = d^2 A(83) / dV(4)dV(31)
      D2A(60) = RCT(83)
C     D2A(61) = d^2 A(85) / dV(22)dV(30)
      D2A(61) = RCT(85)
C     D2A(62) = d^2 A(86) / dV(21)dV(30)
      D2A(62) = RCT(86)
C     D2A(63) = d^2 A(87)/{dV(21)dV(21)}
      D2A(63) = RCT(87)*2
C     D2A(64) = d^2 A(88) / dV(21)dV(22)
      D2A(64) = RCT(88)
C     D2A(65) = d^2 A(89) / dV(27)dV(31)
      D2A(65) = RCT(89)
C     D2A(66) = d^2 A(90) / dV(27)dV(34)
      D2A(66) = RCT(90)
C     D2A(67) = d^2 A(91) / dV(27)dV(33)
      D2A(67) = RCT(91)
C     D2A(68) = d^2 A(93) / dV(26)dV(35)
      D2A(68) = RCT(93)

C     Computation of the Jacobian derivative
C     HESS(1) = d^2 Vdot(1)/{dV(7)dV(35)} = d^2 Vdot(1)/{dV(35)dV(7)}
      HESS(1) = D2A(48)
C     HESS(2) = d^2 Vdot(1)/{dV(13)dV(36)} = d^2 Vdot(1)/{dV(36)dV(13)}
      HESS(2) = 0.1*D2A(45)
C     HESS(3) = d^2 Vdot(1)/{dV(15)dV(35)} = d^2 Vdot(1)/{dV(35)dV(15)}
      HESS(3) = D2A(36)
C     HESS(4) = d^2 Vdot(1)/{dV(21)dV(36)} = d^2 Vdot(1)/{dV(36)dV(21)}
      HESS(4) = D2A(59)
C     HESS(5) = d^2 Vdot(1)/{dV(26)dV(33)} = d^2 Vdot(1)/{dV(33)dV(26)}
      HESS(5) = 0.8*D2A(56)
C     HESS(6) = d^2 Vdot(1)/{dV(26)dV(35)} = d^2 Vdot(1)/{dV(35)dV(26)}
      HESS(6) = 0.8*D2A(68)
C     HESS(7) = d^2 Vdot(1)/{dV(27)dV(33)} = d^2 Vdot(1)/{dV(33)dV(27)}
      HESS(7) = 0.85*D2A(67)
C     HESS(8) = d^2 Vdot(2)/{dV(4)dV(31)} = d^2 Vdot(2)/{dV(31)dV(4)}
      HESS(8) = D2A(60)
C     HESS(9) = d^2 Vdot(4)/{dV(4)dV(31)} = d^2 Vdot(4)/{dV(31)dV(4)}
      HESS(9) = -D2A(60)
C     HESS(10) = d^2 Vdot(5)/{dV(5)dV(31)} = d^2 Vdot(5)/{dV(31)dV(5)}
      HESS(10) = -D2A(23)
C     HESS(11) = d^2 Vdot(5)/{dV(30)dV(30)} = d^2 Vdot(5)/{dV(30)dV(30)}
      HESS(11) = D2A(21)+D2A(22)
C     HESS(12) = d^2 Vdot(6)/{dV(29)dV(35)} = d^2 Vdot(6)/{dV(35)dV(29)}
      HESS(12) = D2A(32)
C     HESS(13) = d^2 Vdot(7)/{dV(7)dV(35)} = d^2 Vdot(7)/{dV(35)dV(7)}
      HESS(13) = -D2A(48)
C     HESS(14) = d^2 Vdot(7)/{dV(16)dV(31)} = d^2 Vdot(7)/{dV(31)dV(16)}
      HESS(14) = 0.4*D2A(46)
C     HESS(15) = d^2 Vdot(7)/{dV(16)dV(33)} = d^2 Vdot(7)/{dV(33)dV(16)}
      HESS(15) = D2A(47)
C     HESS(16) = d^2 Vdot(8)/{dV(8)dV(31)} = d^2 Vdot(8)/{dV(31)dV(8)}
      HESS(16) = -D2A(44)
C     HESS(17) = d^2 Vdot(9)/{dV(33)dV(35)} = d^2 Vdot(9)/{dV(35)dV(33)}
      HESS(17) = D2A(10)
C     HESS(18) = d^2 Vdot(10)/{dV(10)dV(31)} = d^2 Vdot(10)/{dV(31)dV(10)}
      HESS(18) = -D2A(49)
C     HESS(19) = d^2 Vdot(11)/{dV(11)dV(11)} = d^2 Vdot(11)/{dV(11)dV(11)}
      HESS(19) = -2*D2A(15)
C     HESS(20) = d^2 Vdot(11)/{dV(11)dV(31)} = d^2 Vdot(11)/{dV(31)dV(11)}
      HESS(20) = -D2A(14)
C     HESS(21) = d^2 Vdot(11)/{dV(31)dV(36)} = d^2 Vdot(11)/{dV(36)dV(31)}
      HESS(21) = D2A(13)
C     HESS(22) = d^2 Vdot(11)/{dV(35)dV(36)} = d^2 Vdot(11)/{dV(36)dV(35)}
      HESS(22) = 2*D2A(12)
C     HESS(23) = d^2 Vdot(12)/{dV(12)dV(31)} = d^2 Vdot(12)/{dV(31)dV(12)}
      HESS(23) = -D2A(20)
C     HESS(24) = d^2 Vdot(12)/{dV(30)dV(35)} = d^2 Vdot(12)/{dV(35)dV(30)}
      HESS(24) = D2A(19)
C     HESS(25) = d^2 Vdot(13)/{dV(8)dV(31)} = d^2 Vdot(13)/{dV(31)dV(8)}
      HESS(25) = 0.56*D2A(44)
C     HESS(26) = d^2 Vdot(13)/{dV(10)dV(31)} = d^2 Vdot(13)/{dV(31)dV(10)}
      HESS(26) = 0.3*D2A(49)
C     HESS(27) = d^2 Vdot(13)/{dV(13)dV(36)} = d^2 Vdot(13)/{dV(36)dV(13)}
      HESS(27) = -D2A(45)
C     HESS(28) = d^2 Vdot(14)/{dV(14)dV(31)} = d^2 Vdot(14)/{dV(31)dV(14)}
      HESS(28) = -D2A(17)
C     HESS(29) = d^2 Vdot(14)/{dV(16)dV(33)} = d^2 Vdot(14)/{dV(33)dV(16)}
      HESS(29) = D2A(47)
C     HESS(30) = d^2 Vdot(14)/{dV(25)dV(33)} = d^2 Vdot(14)/{dV(33)dV(25)}
      HESS(30) = D2A(27)
C     HESS(31) = d^2 Vdot(14)/{dV(27)dV(33)} = d^2 Vdot(14)/{dV(33)dV(27)}
      HESS(31) = 0.15*D2A(67)
C     HESS(32) = d^2 Vdot(14)/{dV(28)dV(33)} = d^2 Vdot(14)/{dV(33)dV(28)}
      HESS(32) = D2A(30)
C     HESS(33) = d^2 Vdot(14)/{dV(31)dV(35)} = d^2 Vdot(14)/{dV(35)dV(31)}
      HESS(33) = D2A(16)
C     HESS(34) = d^2 Vdot(15)/{dV(15)dV(35)} = d^2 Vdot(15)/{dV(35)dV(15)}
      HESS(34) = -D2A(36)
C     HESS(35) = d^2 Vdot(15)/{dV(24)dV(31)} = d^2 Vdot(15)/{dV(31)dV(24)}
      HESS(35) = 0.76*D2A(35)
C     HESS(36) = d^2 Vdot(16)/{dV(8)dV(31)} = d^2 Vdot(16)/{dV(31)dV(8)}
      HESS(36) = 0.36*D2A(44)
C     HESS(37) = d^2 Vdot(16)/{dV(10)dV(31)} = d^2 Vdot(16)/{dV(31)dV(10)}
      HESS(37) = 0.2*D2A(49)
C     HESS(38) = d^2 Vdot(16)/{dV(16)dV(31)} = d^2 Vdot(16)/{dV(31)dV(16)}
      HESS(38) = -D2A(46)
C     HESS(39) = d^2 Vdot(16)/{dV(16)dV(33)} = d^2 Vdot(16)/{dV(33)dV(16)}
      HESS(39) = -D2A(47)
C     HESS(40) = d^2 Vdot(17)/{dV(10)dV(31)} = d^2 Vdot(17)/{dV(31)dV(10)}
      HESS(40) = 0.8*D2A(49)
C     HESS(41) = d^2 Vdot(17)/{dV(17)dV(31)} = d^2 Vdot(17)/{dV(31)dV(17)}
      HESS(41) = -D2A(52)
C     HESS(42) = d^2 Vdot(17)/{dV(20)dV(34)} = d^2 Vdot(17)/{dV(34)dV(20)}
      HESS(42) = 0.2*D2A(51)
C     HESS(43) = d^2 Vdot(17)/{dV(27)dV(31)} = d^2 Vdot(17)/{dV(31)dV(27)}
      HESS(43) = 0.17*D2A(65)
C     HESS(44) = d^2 Vdot(17)/{dV(27)dV(34)} = d^2 Vdot(17)/{dV(34)dV(27)}
      HESS(44) = 0.85*D2A(66)
C     HESS(45) = d^2 Vdot(18)/{dV(18)dV(31)} = d^2 Vdot(18)/{dV(31)dV(18)}
      HESS(45) = -D2A(24)
C     HESS(46) = d^2 Vdot(18)/{dV(19)dV(32)} = d^2 Vdot(18)/{dV(32)dV(19)}
      HESS(46) = D2A(41)
C     HESS(47) = d^2 Vdot(18)/{dV(19)dV(34)} = d^2 Vdot(18)/{dV(34)dV(19)}
      HESS(47) = 0.42*D2A(43)
C     HESS(48) = d^2 Vdot(18)/{dV(20)dV(31)} = d^2 Vdot(18)/{dV(31)dV(20)}
      HESS(48) = 2*D2A(50)
C     HESS(49) = d^2 Vdot(18)/{dV(20)dV(34)} = d^2 Vdot(18)/{dV(34)dV(20)}
      HESS(49) = 0.69*D2A(51)
C     HESS(50) = d^2 Vdot(18)/{dV(23)dV(32)} = d^2 Vdot(18)/{dV(32)dV(23)}
      HESS(50) = 0.3*D2A(37)
C     HESS(51) = d^2 Vdot(18)/{dV(23)dV(34)} = d^2 Vdot(18)/{dV(34)dV(23)}
      HESS(51) = 0.33*D2A(39)
C     HESS(52) = d^2 Vdot(18)/{dV(25)dV(31)} = d^2 Vdot(18)/{dV(31)dV(25)}
      HESS(52) = D2A(25)
C     HESS(53) = d^2 Vdot(18)/{dV(25)dV(32)} = d^2 Vdot(18)/{dV(32)dV(25)}
      HESS(53) = D2A(26)
C     HESS(54) = d^2 Vdot(18)/{dV(25)dV(33)} = d^2 Vdot(18)/{dV(33)dV(25)}
      HESS(54) = D2A(27)
C     HESS(55) = d^2 Vdot(18)/{dV(26)dV(34)} = d^2 Vdot(18)/{dV(34)dV(26)}
      HESS(55) = 0.07*D2A(55)
C     HESS(56) = d^2 Vdot(18)/{dV(27)dV(31)} = d^2 Vdot(18)/{dV(31)dV(27)}
      HESS(56) = 0.33*D2A(65)
C     HESS(57) = d^2 Vdot(18)/{dV(27)dV(33)} = d^2 Vdot(18)/{dV(33)dV(27)}
      HESS(57) = 0.64*D2A(67)
C     HESS(58) = d^2 Vdot(18)/{dV(27)dV(34)} = d^2 Vdot(18)/{dV(34)dV(27)}
      HESS(58) = 0.23*D2A(66)
C     HESS(59) = d^2 Vdot(19)/{dV(19)dV(31)} = d^2 Vdot(19)/{dV(31)dV(19)}
      HESS(59) = -D2A(42)
C     HESS(60) = d^2 Vdot(19)/{dV(19)dV(32)} = d^2 Vdot(19)/{dV(32)dV(19)}
      HESS(60) = -D2A(41)
C     HESS(61) = d^2 Vdot(19)/{dV(19)dV(34)} = d^2 Vdot(19)/{dV(34)dV(19)}
      HESS(61) = -D2A(43)
C     HESS(62) = d^2 Vdot(20)/{dV(13)dV(36)} = d^2 Vdot(20)/{dV(36)dV(13)}
      HESS(62) = 0.9*D2A(45)
C     HESS(63) = d^2 Vdot(20)/{dV(16)dV(31)} = d^2 Vdot(20)/{dV(31)dV(16)}
      HESS(63) = 0.3*D2A(46)
C     HESS(64) = d^2 Vdot(20)/{dV(20)dV(31)} = d^2 Vdot(20)/{dV(31)dV(20)}
      HESS(64) = -D2A(50)
C     HESS(65) = d^2 Vdot(20)/{dV(20)dV(34)} = d^2 Vdot(20)/{dV(34)dV(20)}
      HESS(65) = -D2A(51)
C     HESS(66) = d^2 Vdot(21)/{dV(21)dV(21)} = d^2 Vdot(21)/{dV(21)dV(21)}
      HESS(66) = -2*D2A(63)
C     HESS(67) = d^2 Vdot(21)/{dV(21)dV(22)} = d^2 Vdot(21)/{dV(22)dV(21)}
      HESS(67) = -D2A(64)
C     HESS(68) = d^2 Vdot(21)/{dV(21)dV(30)} = d^2 Vdot(21)/{dV(30)dV(21)}
      HESS(68) = -D2A(62)
C     HESS(69) = d^2 Vdot(21)/{dV(21)dV(36)} = d^2 Vdot(21)/{dV(36)dV(21)}
      HESS(69) = -D2A(59)
C     HESS(70) = d^2 Vdot(21)/{dV(23)dV(32)} = d^2 Vdot(21)/{dV(32)dV(23)}
      HESS(70) = 0.02*D2A(37)
C     HESS(71) = d^2 Vdot(21)/{dV(23)dV(33)} = d^2 Vdot(21)/{dV(33)dV(23)}
      HESS(71) = 0.09*D2A(40)
C     HESS(72) = d^2 Vdot(21)/{dV(24)dV(31)} = d^2 Vdot(21)/{dV(31)dV(24)}
      HESS(72) = 0.13*D2A(35)
C     HESS(73) = d^2 Vdot(21)/{dV(26)dV(31)} = d^2 Vdot(21)/{dV(31)dV(26)}
      HESS(73) = 0.09*D2A(54)
C     HESS(74) = d^2 Vdot(22)/{dV(8)dV(31)} = d^2 Vdot(22)/{dV(31)dV(8)}
      HESS(74) = 0.08*D2A(44)
C     HESS(75) = d^2 Vdot(22)/{dV(10)dV(31)} = d^2 Vdot(22)/{dV(31)dV(10)}
      HESS(75) = 0.5*D2A(49)
C     HESS(76) = d^2 Vdot(22)/{dV(16)dV(31)} = d^2 Vdot(22)/{dV(31)dV(16)}
      HESS(76) = 0.6*D2A(46)
C     HESS(77) = d^2 Vdot(22)/{dV(17)dV(31)} = d^2 Vdot(22)/{dV(31)dV(17)}
      HESS(77) = D2A(52)
C     HESS(78) = d^2 Vdot(22)/{dV(19)dV(31)} = d^2 Vdot(22)/{dV(31)dV(19)}
      HESS(78) = D2A(42)
C     HESS(79) = d^2 Vdot(22)/{dV(19)dV(32)} = d^2 Vdot(22)/{dV(32)dV(19)}
      HESS(79) = 0.7*D2A(41)
C     HESS(80) = d^2 Vdot(22)/{dV(20)dV(31)} = d^2 Vdot(22)/{dV(31)dV(20)}
      HESS(80) = D2A(50)
C     HESS(81) = d^2 Vdot(22)/{dV(20)dV(34)} = d^2 Vdot(22)/{dV(34)dV(20)}
      HESS(81) = 0.03*D2A(51)
C     HESS(82) = d^2 Vdot(22)/{dV(21)dV(22)} = d^2 Vdot(22)/{dV(22)dV(21)}
      HESS(82) = -D2A(64)
C     HESS(83) = d^2 Vdot(22)/{dV(22)dV(22)} = d^2 Vdot(22)/{dV(22)dV(22)}
      HESS(83) = -2*D2A(58)
C     HESS(84) = d^2 Vdot(22)/{dV(22)dV(30)} = d^2 Vdot(22)/{dV(30)dV(22)}
      HESS(84) = -D2A(61)
C     HESS(85) = d^2 Vdot(22)/{dV(22)dV(36)} = d^2 Vdot(22)/{dV(36)dV(22)}
      HESS(85) = -D2A(57)
C     HESS(86) = d^2 Vdot(22)/{dV(23)dV(31)} = d^2 Vdot(22)/{dV(31)dV(23)}
      HESS(86) = D2A(38)
C     HESS(87) = d^2 Vdot(22)/{dV(23)dV(32)} = d^2 Vdot(22)/{dV(32)dV(23)}
      HESS(87) = 0.28*D2A(37)
C     HESS(88) = d^2 Vdot(22)/{dV(23)dV(33)} = d^2 Vdot(22)/{dV(33)dV(23)}
      HESS(88) = 0.91*D2A(40)
C     HESS(89) = d^2 Vdot(22)/{dV(23)dV(34)} = d^2 Vdot(22)/{dV(34)dV(23)}
      HESS(89) = 0.22*D2A(39)
C     HESS(90) = d^2 Vdot(22)/{dV(24)dV(31)} = d^2 Vdot(22)/{dV(31)dV(24)}
      HESS(90) = 0.87*D2A(35)
C     HESS(91) = d^2 Vdot(22)/{dV(26)dV(31)} = d^2 Vdot(22)/{dV(31)dV(26)}
      HESS(91) = 0.99*D2A(54)
C     HESS(92) = d^2 Vdot(22)/{dV(26)dV(32)} = d^2 Vdot(22)/{dV(32)dV(26)}
      HESS(92) = 0.25*D2A(53)
C     HESS(93) = d^2 Vdot(22)/{dV(26)dV(33)} = d^2 Vdot(22)/{dV(33)dV(26)}
      HESS(93) = D2A(56)
C     HESS(94) = d^2 Vdot(22)/{dV(26)dV(34)} = d^2 Vdot(22)/{dV(34)dV(26)}
      HESS(94) = 0.2*D2A(55)
C     HESS(95) = d^2 Vdot(22)/{dV(26)dV(35)} = d^2 Vdot(22)/{dV(35)dV(26)}
      HESS(95) = D2A(68)
C     HESS(96) = d^2 Vdot(22)/{dV(27)dV(31)} = d^2 Vdot(22)/{dV(31)dV(27)}
      HESS(96) = 0.71*D2A(65)
C     HESS(97) = d^2 Vdot(22)/{dV(27)dV(33)} = d^2 Vdot(22)/{dV(33)dV(27)}
      HESS(97) = 0.08*D2A(67)
C     HESS(98) = d^2 Vdot(22)/{dV(27)dV(34)} = d^2 Vdot(22)/{dV(34)dV(27)}
      HESS(98) = 0.06*D2A(66)
C     HESS(99) = d^2 Vdot(22)/{dV(29)dV(29)} = d^2 Vdot(22)/{dV(29)dV(29)}
      HESS(99) = 2*D2A(33)
C     HESS(100) = d^2 Vdot(22)/{dV(29)dV(30)} = d^2 Vdot(22)/{dV(30)dV(29)}
      HESS(100) = 0.79*D2A(34)
C     HESS(101) = d^2 Vdot(22)/{dV(29)dV(36)} = d^2 Vdot(22)/{dV(36)dV(29)}
      HESS(101) = D2A(31)
C     HESS(102) = d^2 Vdot(23)/{dV(23)dV(31)} = d^2 Vdot(23)/{dV(31)dV(23)}
      HESS(102) = -D2A(38)
C     HESS(103) = d^2 Vdot(23)/{dV(23)dV(32)} = d^2 Vdot(23)/{dV(32)dV(23)}
      HESS(103) = -D2A(37)
C     HESS(104) = d^2 Vdot(23)/{dV(23)dV(33)} = d^2 Vdot(23)/{dV(33)dV(23)}
      HESS(104) = -D2A(40)
C     HESS(105) = d^2 Vdot(23)/{dV(23)dV(34)} = d^2 Vdot(23)/{dV(34)dV(23)}
      HESS(105) = -D2A(39)
C     HESS(106) = d^2 Vdot(24)/{dV(10)dV(31)} = d^2 Vdot(24)/{dV(31)dV(10)}
      HESS(106) = 1.1*D2A(49)
C     HESS(107) = d^2 Vdot(24)/{dV(23)dV(31)} = d^2 Vdot(24)/{dV(31)dV(23)}
      HESS(107) = -D2A(38)
C     HESS(108) = d^2 Vdot(24)/{dV(23)dV(32)} = d^2 Vdot(24)/{dV(32)dV(23)}
      HESS(108) = 0.22*D2A(37)
C     HESS(109) = d^2 Vdot(24)/{dV(23)dV(33)} = d^2 Vdot(24)/{dV(33)dV(23)}
      HESS(109) = -D2A(40)
C     HESS(110) = d^2 Vdot(24)/{dV(23)dV(34)} = d^2 Vdot(24)/{dV(34)dV(23)}
      HESS(110) = -D2A(39)
C     HESS(111) = d^2 Vdot(24)/{dV(24)dV(31)} = d^2 Vdot(24)/{dV(31)dV(24)}
      HESS(111) = -1.11*D2A(35)
C     HESS(112) = d^2 Vdot(24)/{dV(26)dV(32)} = d^2 Vdot(24)/{dV(32)dV(26)}
      HESS(112) = 0.25*D2A(53)
C     HESS(113) = d^2 Vdot(24)/{dV(26)dV(33)} = d^2 Vdot(24)/{dV(33)dV(26)}
      HESS(113) = 2.4*D2A(56)
C     HESS(114) = d^2 Vdot(24)/{dV(26)dV(34)} = d^2 Vdot(24)/{dV(34)dV(26)}
      HESS(114) = 0.35*D2A(55)
C     HESS(115) = d^2 Vdot(24)/{dV(26)dV(35)} = d^2 Vdot(24)/{dV(35)dV(26)}
      HESS(115) = 2.4*D2A(68)
C     HESS(116) = d^2 Vdot(24)/{dV(27)dV(31)} = d^2 Vdot(24)/{dV(31)dV(27)}
      HESS(116) = 1.57*D2A(65)
C     HESS(117) = d^2 Vdot(24)/{dV(27)dV(33)} = d^2 Vdot(24)/{dV(33)dV(27)}
      HESS(117) = 1.28*D2A(67)
C     HESS(118) = d^2 Vdot(24)/{dV(27)dV(34)} = d^2 Vdot(24)/{dV(34)dV(27)}
      HESS(118) = 0.36*D2A(66)
C     HESS(119) = d^2 Vdot(25)/{dV(19)dV(31)} = d^2 Vdot(25)/{dV(31)dV(19)}
      HESS(119) = 1.56*D2A(42)
C     HESS(120) = d^2 Vdot(25)/{dV(19)dV(32)} = d^2 Vdot(25)/{dV(32)dV(19)}
      HESS(120) = D2A(41)
C     HESS(121) = d^2 Vdot(25)/{dV(19)dV(34)} = d^2 Vdot(25)/{dV(34)dV(19)}
      HESS(121) = D2A(43)
C     HESS(122) = d^2 Vdot(25)/{dV(20)dV(31)} = d^2 Vdot(25)/{dV(31)dV(20)}
      HESS(122) = D2A(50)
C     HESS(123) = d^2 Vdot(25)/{dV(20)dV(34)} = d^2 Vdot(25)/{dV(34)dV(20)}
      HESS(123) = 0.7*D2A(51)
C     HESS(124) = d^2 Vdot(25)/{dV(23)dV(31)} = d^2 Vdot(25)/{dV(31)dV(23)}
      HESS(124) = D2A(38)
C     HESS(125) = d^2 Vdot(25)/{dV(23)dV(32)} = d^2 Vdot(25)/{dV(32)dV(23)}
      HESS(125) = 0.2*D2A(37)
C     HESS(126) = d^2 Vdot(25)/{dV(23)dV(33)} = d^2 Vdot(25)/{dV(33)dV(23)}
      HESS(126) = D2A(40)
C     HESS(127) = d^2 Vdot(25)/{dV(23)dV(34)} = d^2 Vdot(25)/{dV(34)dV(23)}
      HESS(127) = 0.74*D2A(39)
C     HESS(128) = d^2 Vdot(25)/{dV(25)dV(31)} = d^2 Vdot(25)/{dV(31)dV(25)}
      HESS(128) = -D2A(25)
C     HESS(129) = d^2 Vdot(25)/{dV(25)dV(32)} = d^2 Vdot(25)/{dV(32)dV(25)}
      HESS(129) = -D2A(26)
C     HESS(130) = d^2 Vdot(25)/{dV(25)dV(33)} = d^2 Vdot(25)/{dV(33)dV(25)}
      HESS(130) = -D2A(27)
C     HESS(131) = d^2 Vdot(25)/{dV(26)dV(31)} = d^2 Vdot(25)/{dV(31)dV(26)}
      HESS(131) = 0.63*D2A(54)
C     HESS(132) = d^2 Vdot(25)/{dV(26)dV(32)} = d^2 Vdot(25)/{dV(32)dV(26)}
      HESS(132) = 0.5*D2A(53)
C     HESS(133) = d^2 Vdot(25)/{dV(26)dV(34)} = d^2 Vdot(25)/{dV(34)dV(26)}
      HESS(133) = 0.6*D2A(55)
C     HESS(134) = d^2 Vdot(25)/{dV(27)dV(31)} = d^2 Vdot(25)/{dV(31)dV(27)}
      HESS(134) = 0.17*D2A(65)
C     HESS(135) = d^2 Vdot(25)/{dV(27)dV(33)} = d^2 Vdot(25)/{dV(33)dV(27)}
      HESS(135) = 0.28*D2A(67)
C     HESS(136) = d^2 Vdot(25)/{dV(27)dV(34)} = d^2 Vdot(25)/{dV(34)dV(27)}
      HESS(136) = 0.15*D2A(66)
C     HESS(137) = d^2 Vdot(25)/{dV(29)dV(29)} = d^2 Vdot(25)/{dV(29)dV(29)}
      HESS(137) = 2*D2A(33)
C     HESS(138) = d^2 Vdot(25)/{dV(29)dV(30)} = d^2 Vdot(25)/{dV(30)dV(29)}
      HESS(138) = 0.79*D2A(34)
C     HESS(139) = d^2 Vdot(25)/{dV(29)dV(36)} = d^2 Vdot(25)/{dV(36)dV(29)}
      HESS(139) = D2A(31)
C     HESS(140) = d^2 Vdot(26)/{dV(26)dV(31)} = d^2 Vdot(26)/{dV(31)dV(26)}
      HESS(140) = -D2A(54)
C     HESS(141) = d^2 Vdot(26)/{dV(26)dV(32)} = d^2 Vdot(26)/{dV(32)dV(26)}
      HESS(141) = -D2A(53)
C     HESS(142) = d^2 Vdot(26)/{dV(26)dV(33)} = d^2 Vdot(26)/{dV(33)dV(26)}
      HESS(142) = -D2A(56)
C     HESS(143) = d^2 Vdot(26)/{dV(26)dV(34)} = d^2 Vdot(26)/{dV(34)dV(26)}
      HESS(143) = -D2A(55)
C     HESS(144) = d^2 Vdot(26)/{dV(26)dV(35)} = d^2 Vdot(26)/{dV(35)dV(26)}
      HESS(144) = -D2A(68)
C     HESS(145) = d^2 Vdot(27)/{dV(26)dV(31)} = d^2 Vdot(27)/{dV(31)dV(26)}
      HESS(145) = 0.91*D2A(54)
C     HESS(146) = d^2 Vdot(27)/{dV(26)dV(32)} = d^2 Vdot(27)/{dV(32)dV(26)}
      HESS(146) = 0.75*D2A(53)
C     HESS(147) = d^2 Vdot(27)/{dV(26)dV(33)} = d^2 Vdot(27)/{dV(33)dV(26)}
      HESS(147) = 0.2*D2A(56)
C     HESS(148) = d^2 Vdot(27)/{dV(26)dV(34)} = d^2 Vdot(27)/{dV(34)dV(26)}
      HESS(148) = 0.65*D2A(55)
C     HESS(149) = d^2 Vdot(27)/{dV(26)dV(35)} = d^2 Vdot(27)/{dV(35)dV(26)}
      HESS(149) = 0.2*D2A(68)
C     HESS(150) = d^2 Vdot(27)/{dV(27)dV(31)} = d^2 Vdot(27)/{dV(31)dV(27)}
      HESS(150) = -D2A(65)
C     HESS(151) = d^2 Vdot(27)/{dV(27)dV(33)} = d^2 Vdot(27)/{dV(33)dV(27)}
      HESS(151) = -D2A(67)
C     HESS(152) = d^2 Vdot(27)/{dV(27)dV(34)} = d^2 Vdot(27)/{dV(34)dV(27)}
      HESS(152) = -D2A(66)
C     HESS(153) = d^2 Vdot(28)/{dV(19)dV(31)} = d^2 Vdot(28)/{dV(31)dV(19)}
      HESS(153) = 0.22*D2A(42)
C     HESS(154) = d^2 Vdot(28)/{dV(20)dV(34)} = d^2 Vdot(28)/{dV(34)dV(20)}
      HESS(154) = 0.03*D2A(51)
C     HESS(155) = d^2 Vdot(28)/{dV(23)dV(31)} = d^2 Vdot(28)/{dV(31)dV(23)}
      HESS(155) = D2A(38)
C     HESS(156) = d^2 Vdot(28)/{dV(23)dV(32)} = d^2 Vdot(28)/{dV(32)dV(23)}
      HESS(156) = 0.63*D2A(37)
C     HESS(157) = d^2 Vdot(28)/{dV(23)dV(33)} = d^2 Vdot(28)/{dV(33)dV(23)}
      HESS(157) = D2A(40)
C     HESS(158) = d^2 Vdot(28)/{dV(23)dV(34)} = d^2 Vdot(28)/{dV(34)dV(23)}
      HESS(158) = 0.5*D2A(39)
C     HESS(159) = d^2 Vdot(28)/{dV(24)dV(31)} = d^2 Vdot(28)/{dV(31)dV(24)}
      HESS(159) = 0.11*D2A(35)
C     HESS(160) = d^2 Vdot(28)/{dV(26)dV(33)} = d^2 Vdot(28)/{dV(33)dV(26)}
      HESS(160) = 0.8*D2A(56)
C     HESS(161) = d^2 Vdot(28)/{dV(26)dV(34)} = d^2 Vdot(28)/{dV(34)dV(26)}
      HESS(161) = 0.15*D2A(55)
C     HESS(162) = d^2 Vdot(28)/{dV(26)dV(35)} = d^2 Vdot(28)/{dV(35)dV(26)}
      HESS(162) = 0.8*D2A(68)
C     HESS(163) = d^2 Vdot(28)/{dV(27)dV(31)} = d^2 Vdot(28)/{dV(31)dV(27)}
      HESS(163) = 0.27*D2A(65)
C     HESS(164) = d^2 Vdot(28)/{dV(27)dV(33)} = d^2 Vdot(28)/{dV(33)dV(27)}
      HESS(164) = 0.36*D2A(67)
C     HESS(165) = d^2 Vdot(28)/{dV(27)dV(34)} = d^2 Vdot(28)/{dV(34)dV(27)}
      HESS(165) = 0.02*D2A(66)
C     HESS(166) = d^2 Vdot(28)/{dV(28)dV(31)} = d^2 Vdot(28)/{dV(31)dV(28)}
      HESS(166) = -D2A(29)
C     HESS(167) = d^2 Vdot(28)/{dV(28)dV(32)} = d^2 Vdot(28)/{dV(32)dV(28)}
      HESS(167) = -D2A(28)
C     HESS(168) = d^2 Vdot(28)/{dV(28)dV(33)} = d^2 Vdot(28)/{dV(33)dV(28)}
      HESS(168) = -D2A(30)
C     HESS(169) = d^2 Vdot(29)/{dV(17)dV(31)} = d^2 Vdot(29)/{dV(31)dV(17)}
      HESS(169) = D2A(52)
C     HESS(170) = d^2 Vdot(29)/{dV(20)dV(31)} = d^2 Vdot(29)/{dV(31)dV(20)}
      HESS(170) = D2A(50)
C     HESS(171) = d^2 Vdot(29)/{dV(20)dV(34)} = d^2 Vdot(29)/{dV(34)dV(20)}
      HESS(171) = 0.62*D2A(51)
C     HESS(172) = d^2 Vdot(29)/{dV(26)dV(32)} = d^2 Vdot(29)/{dV(32)dV(26)}
      HESS(172) = 0.25*D2A(53)
C     HESS(173) = d^2 Vdot(29)/{dV(26)dV(34)} = d^2 Vdot(29)/{dV(34)dV(26)}
      HESS(173) = 0.2*D2A(55)
C     HESS(174) = d^2 Vdot(29)/{dV(27)dV(31)} = d^2 Vdot(29)/{dV(31)dV(27)}
      HESS(174) = 0.5*D2A(65)
C     HESS(175) = d^2 Vdot(29)/{dV(27)dV(33)} = d^2 Vdot(29)/{dV(33)dV(27)}
      HESS(175) = 0.08*D2A(67)
C     HESS(176) = d^2 Vdot(29)/{dV(27)dV(34)} = d^2 Vdot(29)/{dV(34)dV(27)}
      HESS(176) = 0.11*D2A(66)
C     HESS(177) = d^2 Vdot(29)/{dV(28)dV(31)} = d^2 Vdot(29)/{dV(31)dV(28)}
      HESS(177) = D2A(29)
C     HESS(178) = d^2 Vdot(29)/{dV(28)dV(32)} = d^2 Vdot(29)/{dV(32)dV(28)}
      HESS(178) = D2A(28)
C     HESS(179) = d^2 Vdot(29)/{dV(28)dV(33)} = d^2 Vdot(29)/{dV(33)dV(28)}
      HESS(179) = D2A(30)
C     HESS(180) = d^2 Vdot(29)/{dV(29)dV(29)} = d^2 Vdot(29)/{dV(29)dV(29)}
      HESS(180) = -2*D2A(33)
C     HESS(181) = d^2 Vdot(29)/{dV(29)dV(30)} = d^2 Vdot(29)/{dV(30)dV(29)}
      HESS(181) = -D2A(34)
C     HESS(182) = d^2 Vdot(29)/{dV(29)dV(35)} = d^2 Vdot(29)/{dV(35)dV(29)}
      HESS(182) = -D2A(32)
C     HESS(183) = d^2 Vdot(29)/{dV(29)dV(36)} = d^2 Vdot(29)/{dV(36)dV(29)}
      HESS(183) = -D2A(31)
C     HESS(184) = d^2 Vdot(30)/{dV(4)dV(31)} = d^2 Vdot(30)/{dV(31)dV(4)}
      HESS(184) = D2A(60)
C     HESS(185) = d^2 Vdot(30)/{dV(5)dV(31)} = d^2 Vdot(30)/{dV(31)dV(5)}
      HESS(185) = D2A(23)
C     HESS(186) = d^2 Vdot(30)/{dV(8)dV(31)} = d^2 Vdot(30)/{dV(31)dV(8)}
      HESS(186) = 0.44*D2A(44)
C     HESS(187) = d^2 Vdot(30)/{dV(10)dV(31)} = d^2 Vdot(30)/{dV(31)dV(10)}
      HESS(187) = 0.7*D2A(49)
C     HESS(188) = d^2 Vdot(30)/{dV(13)dV(36)} = d^2 Vdot(30)/{dV(36)dV(13)}
      HESS(188) = 0.9*D2A(45)
C     HESS(189) = d^2 Vdot(30)/{dV(16)dV(31)} = d^2 Vdot(30)/{dV(31)dV(16)}
      HESS(189) = 0.6*D2A(46)
C     HESS(190) = d^2 Vdot(30)/{dV(18)dV(31)} = d^2 Vdot(30)/{dV(31)dV(18)}
      HESS(190) = D2A(24)
C     HESS(191) = d^2 Vdot(30)/{dV(19)dV(31)} = d^2 Vdot(30)/{dV(31)dV(19)}
      HESS(191) = D2A(42)
C     HESS(192) = d^2 Vdot(30)/{dV(19)dV(32)} = d^2 Vdot(30)/{dV(32)dV(19)}
      HESS(192) = 1.7*D2A(41)
C     HESS(193) = d^2 Vdot(30)/{dV(19)dV(34)} = d^2 Vdot(30)/{dV(34)dV(19)}
      HESS(193) = 0.12*D2A(43)
C     HESS(194) = d^2 Vdot(30)/{dV(20)dV(31)} = d^2 Vdot(30)/{dV(31)dV(20)}
      HESS(194) = 2*D2A(50)
C     HESS(195) = d^2 Vdot(30)/{dV(20)dV(34)} = d^2 Vdot(30)/{dV(34)dV(20)}
      HESS(195) = 0.76*D2A(51)
C     HESS(196) = d^2 Vdot(30)/{dV(21)dV(30)} = d^2 Vdot(30)/{dV(30)dV(21)}
      HESS(196) = -D2A(62)
C     HESS(197) = d^2 Vdot(30)/{dV(22)dV(30)} = d^2 Vdot(30)/{dV(30)dV(22)}
      HESS(197) = -D2A(61)
C     HESS(198) = d^2 Vdot(30)/{dV(23)dV(31)} = d^2 Vdot(30)/{dV(31)dV(23)}
      HESS(198) = D2A(38)
C     HESS(199) = d^2 Vdot(30)/{dV(23)dV(32)} = d^2 Vdot(30)/{dV(32)dV(23)}
      HESS(199) = 0.38*D2A(37)
C     HESS(200) = d^2 Vdot(30)/{dV(23)dV(34)} = d^2 Vdot(30)/{dV(34)dV(23)}
      HESS(200) = 0.44*D2A(39)
C     HESS(201) = d^2 Vdot(30)/{dV(24)dV(31)} = d^2 Vdot(30)/{dV(31)dV(24)}
      HESS(201) = 0.11*D2A(35)
C     HESS(202) = d^2 Vdot(30)/{dV(25)dV(31)} = d^2 Vdot(30)/{dV(31)dV(25)}
      HESS(202) = D2A(25)
C     HESS(203) = d^2 Vdot(30)/{dV(25)dV(32)} = d^2 Vdot(30)/{dV(32)dV(25)}
      HESS(203) = D2A(26)
C     HESS(204) = d^2 Vdot(30)/{dV(25)dV(33)} = d^2 Vdot(30)/{dV(33)dV(25)}
      HESS(204) = D2A(27)
C     HESS(205) = d^2 Vdot(30)/{dV(26)dV(31)} = d^2 Vdot(30)/{dV(31)dV(26)}
      HESS(205) = 0.91*D2A(54)
C     HESS(206) = d^2 Vdot(30)/{dV(26)dV(32)} = d^2 Vdot(30)/{dV(32)dV(26)}
      HESS(206) = 0.25*D2A(53)
C     HESS(207) = d^2 Vdot(30)/{dV(26)dV(33)} = d^2 Vdot(30)/{dV(33)dV(26)}
      HESS(207) = 0.8*D2A(56)
C     HESS(208) = d^2 Vdot(30)/{dV(26)dV(34)} = d^2 Vdot(30)/{dV(34)dV(26)}
      HESS(208) = 0.07*D2A(55)
C     HESS(209) = d^2 Vdot(30)/{dV(26)dV(35)} = d^2 Vdot(30)/{dV(35)dV(26)}
      HESS(209) = 0.8*D2A(68)
C     HESS(210) = d^2 Vdot(30)/{dV(27)dV(31)} = d^2 Vdot(30)/{dV(31)dV(27)}
      HESS(210) = 0.5*D2A(65)
C     HESS(211) = d^2 Vdot(30)/{dV(27)dV(33)} = d^2 Vdot(30)/{dV(33)dV(27)}
      HESS(211) = 0.93*D2A(67)
C     HESS(212) = d^2 Vdot(30)/{dV(27)dV(34)} = d^2 Vdot(30)/{dV(34)dV(27)}
      HESS(212) = 0.15*D2A(66)
C     HESS(213) = d^2 Vdot(30)/{dV(29)dV(29)} = d^2 Vdot(30)/{dV(29)dV(29)}
      HESS(213) = 2*D2A(33)
C     HESS(214) = d^2 Vdot(30)/{dV(29)dV(30)} = d^2 Vdot(30)/{dV(30)dV(29)}
      HESS(214) = -0.21*D2A(34)
C     HESS(215) = d^2 Vdot(30)/{dV(29)dV(36)} = d^2 Vdot(30)/{dV(36)dV(29)}
      HESS(215) = D2A(31)
C     HESS(216) = d^2 Vdot(30)/{dV(30)dV(30)} = d^2 Vdot(30)/{dV(30)dV(30)}
      HESS(216) = -2*D2A(21)-2*D2A(22)
C     HESS(217) = d^2 Vdot(30)/{dV(30)dV(34)} = d^2 Vdot(30)/{dV(34)dV(30)}
      HESS(217) = -D2A(7)
C     HESS(218) = d^2 Vdot(30)/{dV(30)dV(35)} = d^2 Vdot(30)/{dV(35)dV(30)}
      HESS(218) = -D2A(19)
C     HESS(219) = d^2 Vdot(30)/{dV(30)dV(36)} = d^2 Vdot(30)/{dV(36)dV(30)}
      HESS(219) = -D2A(18)
C     HESS(220) = d^2 Vdot(30)/{dV(31)dV(34)} = d^2 Vdot(30)/{dV(34)dV(31)}
      HESS(220) = D2A(6)
C     HESS(221) = d^2 Vdot(31)/{dV(4)dV(31)} = d^2 Vdot(31)/{dV(31)dV(4)}
      HESS(221) = -D2A(60)
C     HESS(222) = d^2 Vdot(31)/{dV(5)dV(31)} = d^2 Vdot(31)/{dV(31)dV(5)}
      HESS(222) = -D2A(23)
C     HESS(223) = d^2 Vdot(31)/{dV(8)dV(31)} = d^2 Vdot(31)/{dV(31)dV(8)}
      HESS(223) = -D2A(44)
C     HESS(224) = d^2 Vdot(31)/{dV(10)dV(31)} = d^2 Vdot(31)/{dV(31)dV(10)}
      HESS(224) = -D2A(49)
C     HESS(225) = d^2 Vdot(31)/{dV(11)dV(31)} = d^2 Vdot(31)/{dV(31)dV(11)}
      HESS(225) = -D2A(14)
C     HESS(226) = d^2 Vdot(31)/{dV(12)dV(31)} = d^2 Vdot(31)/{dV(31)dV(12)}
      HESS(226) = -D2A(20)
C     HESS(227) = d^2 Vdot(31)/{dV(14)dV(31)} = d^2 Vdot(31)/{dV(31)dV(14)}
      HESS(227) = -D2A(17)
C     HESS(228) = d^2 Vdot(31)/{dV(16)dV(31)} = d^2 Vdot(31)/{dV(31)dV(16)}
      HESS(228) = -D2A(46)
C     HESS(229) = d^2 Vdot(31)/{dV(17)dV(31)} = d^2 Vdot(31)/{dV(31)dV(17)}
      HESS(229) = -D2A(52)
C     HESS(230) = d^2 Vdot(31)/{dV(18)dV(31)} = d^2 Vdot(31)/{dV(31)dV(18)}
      HESS(230) = -D2A(24)
C     HESS(231) = d^2 Vdot(31)/{dV(19)dV(31)} = d^2 Vdot(31)/{dV(31)dV(19)}
      HESS(231) = -D2A(42)
C     HESS(232) = d^2 Vdot(31)/{dV(19)dV(32)} = d^2 Vdot(31)/{dV(32)dV(19)}
      HESS(232) = 0.3*D2A(41)
C     HESS(233) = d^2 Vdot(31)/{dV(20)dV(31)} = d^2 Vdot(31)/{dV(31)dV(20)}
      HESS(233) = -D2A(50)
C     HESS(234) = d^2 Vdot(31)/{dV(20)dV(34)} = d^2 Vdot(31)/{dV(34)dV(20)}
      HESS(234) = 0.08*D2A(51)
C     HESS(235) = d^2 Vdot(31)/{dV(23)dV(31)} = d^2 Vdot(31)/{dV(31)dV(23)}
      HESS(235) = -D2A(38)
C     HESS(236) = d^2 Vdot(31)/{dV(23)dV(32)} = d^2 Vdot(31)/{dV(32)dV(23)}
      HESS(236) = 0.2*D2A(37)
C     HESS(237) = d^2 Vdot(31)/{dV(23)dV(34)} = d^2 Vdot(31)/{dV(34)dV(23)}
      HESS(237) = 0.1*D2A(39)
C     HESS(238) = d^2 Vdot(31)/{dV(24)dV(31)} = d^2 Vdot(31)/{dV(31)dV(24)}
      HESS(238) = -D2A(35)
C     HESS(239) = d^2 Vdot(31)/{dV(25)dV(31)} = d^2 Vdot(31)/{dV(31)dV(25)}
      HESS(239) = -D2A(25)
C     HESS(240) = d^2 Vdot(31)/{dV(25)dV(32)} = d^2 Vdot(31)/{dV(32)dV(25)}
      HESS(240) = D2A(26)
C     HESS(241) = d^2 Vdot(31)/{dV(26)dV(31)} = d^2 Vdot(31)/{dV(31)dV(26)}
      HESS(241) = -D2A(54)
C     HESS(242) = d^2 Vdot(31)/{dV(26)dV(34)} = d^2 Vdot(31)/{dV(34)dV(26)}
      HESS(242) = 0.27*D2A(55)
C     HESS(243) = d^2 Vdot(31)/{dV(27)dV(31)} = d^2 Vdot(31)/{dV(31)dV(27)}
      HESS(243) = -D2A(65)
C     HESS(244) = d^2 Vdot(31)/{dV(27)dV(34)} = d^2 Vdot(31)/{dV(34)dV(27)}
      HESS(244) = 0.27*D2A(66)
C     HESS(245) = d^2 Vdot(31)/{dV(28)dV(31)} = d^2 Vdot(31)/{dV(31)dV(28)}
      HESS(245) = -D2A(29)
C     HESS(246) = d^2 Vdot(31)/{dV(28)dV(32)} = d^2 Vdot(31)/{dV(32)dV(28)}
      HESS(246) = D2A(28)
C     HESS(247) = d^2 Vdot(31)/{dV(29)dV(30)} = d^2 Vdot(31)/{dV(30)dV(29)}
      HESS(247) = 0.79*D2A(34)
C     HESS(248) = d^2 Vdot(31)/{dV(30)dV(34)} = d^2 Vdot(31)/{dV(34)dV(30)}
      HESS(248) = D2A(7)
C     HESS(249) = d^2 Vdot(31)/{dV(30)dV(36)} = d^2 Vdot(31)/{dV(36)dV(30)}
      HESS(249) = D2A(18)
C     HESS(250) = d^2 Vdot(31)/{dV(31)dV(34)} = d^2 Vdot(31)/{dV(34)dV(31)}
      HESS(250) = -D2A(6)
C     HESS(251) = d^2 Vdot(31)/{dV(31)dV(35)} = d^2 Vdot(31)/{dV(35)dV(31)}
      HESS(251) = -D2A(16)
C     HESS(252) = d^2 Vdot(31)/{dV(31)dV(36)} = d^2 Vdot(31)/{dV(36)dV(31)}
      HESS(252) = -D2A(13)
C     HESS(253) = d^2 Vdot(32)/{dV(19)dV(32)} = d^2 Vdot(32)/{dV(32)dV(19)}
      HESS(253) = -D2A(41)
C     HESS(254) = d^2 Vdot(32)/{dV(23)dV(32)} = d^2 Vdot(32)/{dV(32)dV(23)}
      HESS(254) = -D2A(37)
C     HESS(255) = d^2 Vdot(32)/{dV(25)dV(32)} = d^2 Vdot(32)/{dV(32)dV(25)}
      HESS(255) = -D2A(26)
C     HESS(256) = d^2 Vdot(32)/{dV(26)dV(32)} = d^2 Vdot(32)/{dV(32)dV(26)}
      HESS(256) = -D2A(53)
C     HESS(257) = d^2 Vdot(32)/{dV(28)dV(32)} = d^2 Vdot(32)/{dV(32)dV(28)}
      HESS(257) = -D2A(28)
C     HESS(258) = d^2 Vdot(32)/{dV(32)dV(35)} = d^2 Vdot(32)/{dV(35)dV(32)}
      HESS(258) = -D2A(2)-D2A(3)
C     HESS(259) = d^2 Vdot(32)/{dV(32)dV(36)} = d^2 Vdot(32)/{dV(36)dV(32)}
      HESS(259) = -D2A(4)
C     HESS(260) = d^2 Vdot(33)/{dV(14)dV(31)} = d^2 Vdot(33)/{dV(31)dV(14)}
      HESS(260) = D2A(17)
C     HESS(261) = d^2 Vdot(33)/{dV(16)dV(33)} = d^2 Vdot(33)/{dV(33)dV(16)}
      HESS(261) = -D2A(47)
C     HESS(262) = d^2 Vdot(33)/{dV(23)dV(33)} = d^2 Vdot(33)/{dV(33)dV(23)}
      HESS(262) = -D2A(40)
C     HESS(263) = d^2 Vdot(33)/{dV(25)dV(33)} = d^2 Vdot(33)/{dV(33)dV(25)}
      HESS(263) = -D2A(27)
C     HESS(264) = d^2 Vdot(33)/{dV(26)dV(33)} = d^2 Vdot(33)/{dV(33)dV(26)}
      HESS(264) = -D2A(56)
C     HESS(265) = d^2 Vdot(33)/{dV(27)dV(33)} = d^2 Vdot(33)/{dV(33)dV(27)}
      HESS(265) = -D2A(67)
C     HESS(266) = d^2 Vdot(33)/{dV(28)dV(33)} = d^2 Vdot(33)/{dV(33)dV(28)}
      HESS(266) = -D2A(30)
C     HESS(267) = d^2 Vdot(33)/{dV(32)dV(35)} = d^2 Vdot(33)/{dV(35)dV(32)}
      HESS(267) = D2A(3)
C     HESS(268) = d^2 Vdot(33)/{dV(33)dV(35)} = d^2 Vdot(33)/{dV(35)dV(33)}
      HESS(268) = -D2A(9)-D2A(10)
C     HESS(269) = d^2 Vdot(33)/{dV(33)dV(36)} = d^2 Vdot(33)/{dV(36)dV(33)}
      HESS(269) = -D2A(8)
C     HESS(270) = d^2 Vdot(33)/{dV(34)dV(35)} = d^2 Vdot(33)/{dV(35)dV(34)}
      HESS(270) = D2A(5)
C     HESS(271) = d^2 Vdot(34)/{dV(19)dV(34)} = d^2 Vdot(34)/{dV(34)dV(19)}
      HESS(271) = -D2A(43)
C     HESS(272) = d^2 Vdot(34)/{dV(20)dV(34)} = d^2 Vdot(34)/{dV(34)dV(20)}
      HESS(272) = -D2A(51)
C     HESS(273) = d^2 Vdot(34)/{dV(23)dV(34)} = d^2 Vdot(34)/{dV(34)dV(23)}
      HESS(273) = -D2A(39)
C     HESS(274) = d^2 Vdot(34)/{dV(26)dV(34)} = d^2 Vdot(34)/{dV(34)dV(26)}
      HESS(274) = -D2A(55)
C     HESS(275) = d^2 Vdot(34)/{dV(27)dV(34)} = d^2 Vdot(34)/{dV(34)dV(27)}
      HESS(275) = -D2A(66)
C     HESS(276) = d^2 Vdot(34)/{dV(30)dV(34)} = d^2 Vdot(34)/{dV(34)dV(30)}
      HESS(276) = -D2A(7)
C     HESS(277) = d^2 Vdot(34)/{dV(31)dV(34)} = d^2 Vdot(34)/{dV(34)dV(31)}
      HESS(277) = -D2A(6)
C     HESS(278) = d^2 Vdot(34)/{dV(34)dV(35)} = d^2 Vdot(34)/{dV(35)dV(34)}
      HESS(278) = -D2A(5)
C     HESS(279) = d^2 Vdot(34)/{dV(34)dV(36)} = d^2 Vdot(34)/{dV(36)dV(34)}
      HESS(279) = -D2A(1)
C     HESS(280) = d^2 Vdot(35)/{dV(7)dV(35)} = d^2 Vdot(35)/{dV(35)dV(7)}
      HESS(280) = -D2A(48)
C     HESS(281) = d^2 Vdot(35)/{dV(11)dV(11)} = d^2 Vdot(35)/{dV(11)dV(11)}
      HESS(281) = D2A(15)
C     HESS(282) = d^2 Vdot(35)/{dV(11)dV(31)} = d^2 Vdot(35)/{dV(31)dV(11)}
      HESS(282) = D2A(14)
C     HESS(283) = d^2 Vdot(35)/{dV(12)dV(31)} = d^2 Vdot(35)/{dV(31)dV(12)}
      HESS(283) = D2A(20)
C     HESS(284) = d^2 Vdot(35)/{dV(13)dV(36)} = d^2 Vdot(35)/{dV(36)dV(13)}
      HESS(284) = 0.9*D2A(45)
C     HESS(285) = d^2 Vdot(35)/{dV(15)dV(35)} = d^2 Vdot(35)/{dV(35)dV(15)}
      HESS(285) = -D2A(36)
C     HESS(286) = d^2 Vdot(35)/{dV(22)dV(36)} = d^2 Vdot(35)/{dV(36)dV(22)}
      HESS(286) = D2A(57)
C     HESS(287) = d^2 Vdot(35)/{dV(23)dV(33)} = d^2 Vdot(35)/{dV(33)dV(23)}
      HESS(287) = D2A(40)
C     HESS(288) = d^2 Vdot(35)/{dV(26)dV(33)} = d^2 Vdot(35)/{dV(33)dV(26)}
      HESS(288) = 0.2*D2A(56)
C     HESS(289) = d^2 Vdot(35)/{dV(26)dV(35)} = d^2 Vdot(35)/{dV(35)dV(26)}
      HESS(289) = -D2A(68)
C     HESS(290) = d^2 Vdot(35)/{dV(29)dV(35)} = d^2 Vdot(35)/{dV(35)dV(29)}
      HESS(290) = -D2A(32)
C     HESS(291) = d^2 Vdot(35)/{dV(29)dV(36)} = d^2 Vdot(35)/{dV(36)dV(29)}
      HESS(291) = D2A(31)
C     HESS(292) = d^2 Vdot(35)/{dV(30)dV(35)} = d^2 Vdot(35)/{dV(35)dV(30)}
      HESS(292) = -D2A(19)
C     HESS(293) = d^2 Vdot(35)/{dV(30)dV(36)} = d^2 Vdot(35)/{dV(36)dV(30)}
      HESS(293) = D2A(18)
C     HESS(294) = d^2 Vdot(35)/{dV(31)dV(35)} = d^2 Vdot(35)/{dV(35)dV(31)}
      HESS(294) = -D2A(16)
C     HESS(295) = d^2 Vdot(35)/{dV(32)dV(35)} = d^2 Vdot(35)/{dV(35)dV(32)}
      HESS(295) = -D2A(2)-D2A(3)
C     HESS(296) = d^2 Vdot(35)/{dV(32)dV(36)} = d^2 Vdot(35)/{dV(36)dV(32)}
      HESS(296) = D2A(4)
C     HESS(297) = d^2 Vdot(35)/{dV(33)dV(35)} = d^2 Vdot(35)/{dV(35)dV(33)}
      HESS(297) = -D2A(10)
C     HESS(298) = d^2 Vdot(35)/{dV(33)dV(36)} = d^2 Vdot(35)/{dV(36)dV(33)}
      HESS(298) = 2*D2A(8)
C     HESS(299) = d^2 Vdot(35)/{dV(34)dV(35)} = d^2 Vdot(35)/{dV(35)dV(34)}
      HESS(299) = -D2A(5)
C     HESS(300) = d^2 Vdot(35)/{dV(34)dV(36)} = d^2 Vdot(35)/{dV(36)dV(34)}
      HESS(300) = D2A(1)
C     HESS(301) = d^2 Vdot(35)/{dV(35)dV(36)} = d^2 Vdot(35)/{dV(36)dV(35)}
      HESS(301) = -D2A(12)
C     HESS(302) = d^2 Vdot(35)/{dV(36)dV(36)} = d^2 Vdot(35)/{dV(36)dV(36)}
      HESS(302) = 2*D2A(11)
C     HESS(303) = d^2 Vdot(36)/{dV(11)dV(11)} = d^2 Vdot(36)/{dV(11)dV(11)}
      HESS(303) = D2A(15)
C     HESS(304) = d^2 Vdot(36)/{dV(13)dV(36)} = d^2 Vdot(36)/{dV(36)dV(13)}
      HESS(304) = -D2A(45)
C     HESS(305) = d^2 Vdot(36)/{dV(21)dV(36)} = d^2 Vdot(36)/{dV(36)dV(21)}
      HESS(305) = -D2A(59)
C     HESS(306) = d^2 Vdot(36)/{dV(22)dV(36)} = d^2 Vdot(36)/{dV(36)dV(22)}
      HESS(306) = -D2A(57)
C     HESS(307) = d^2 Vdot(36)/{dV(26)dV(35)} = d^2 Vdot(36)/{dV(35)dV(26)}
      HESS(307) = 0.2*D2A(68)
C     HESS(308) = d^2 Vdot(36)/{dV(29)dV(36)} = d^2 Vdot(36)/{dV(36)dV(29)}
      HESS(308) = -D2A(31)
C     HESS(309) = d^2 Vdot(36)/{dV(30)dV(36)} = d^2 Vdot(36)/{dV(36)dV(30)}
      HESS(309) = -D2A(18)
C     HESS(310) = d^2 Vdot(36)/{dV(31)dV(36)} = d^2 Vdot(36)/{dV(36)dV(31)}
      HESS(310) = -D2A(13)
C     HESS(311) = d^2 Vdot(36)/{dV(32)dV(35)} = d^2 Vdot(36)/{dV(35)dV(32)}
      HESS(311) = D2A(2)
C     HESS(312) = d^2 Vdot(36)/{dV(32)dV(36)} = d^2 Vdot(36)/{dV(36)dV(32)}
      HESS(312) = -D2A(4)
C     HESS(313) = d^2 Vdot(36)/{dV(33)dV(35)} = d^2 Vdot(36)/{dV(35)dV(33)}
      HESS(313) = D2A(9)
C     HESS(314) = d^2 Vdot(36)/{dV(33)dV(36)} = d^2 Vdot(36)/{dV(36)dV(33)}
      HESS(314) = -D2A(8)
C     HESS(315) = d^2 Vdot(36)/{dV(34)dV(36)} = d^2 Vdot(36)/{dV(36)dV(34)}
      HESS(315) = -D2A(1)
C     HESS(316) = d^2 Vdot(36)/{dV(35)dV(36)} = d^2 Vdot(36)/{dV(36)dV(35)}
      HESS(316) = -D2A(12)
C     HESS(317) = d^2 Vdot(36)/{dV(36)dV(36)} = d^2 Vdot(36)/{dV(36)dV(36)}
      HESS(317) = -2*D2A(11)
      
      END SUBROUTINE Hessian

C     End of Hessian function
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     
C     HessTR_Vec - Hessian transposed times user vectors
C     Arguments :
C     HESS      - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
C     U1        - User vector
C     U2        - User vector
C     HTU       - Transposed Hessian times user vectors: (Hess x U2)^T * U1 = [d (Jac^T*U1)/d Var] * U2
C     
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      SUBROUTINE HessTR_Vec ( HESS, U1, U2, HTU )

C     HESS - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
      REAL(kind=dp) :: HESS(NHESS)
C     U1 - User vector
      REAL(kind=dp) :: U1(KPPNSP)
C     U2 - User vector
      REAL(kind=dp) :: U2(KPPNSP)
C     HTU - Transposed Hessian times user vectors: (Hess x U2)^T * U1 = [d (Jac^T*U1)/d Var] * U2
      REAL(kind=dp) :: HTU(KPPNSP)

C     Compute the vector HTU =(Hess x U2)^T * U1 = d (Jac^T*U1)/d Var * U2
      HTU(1) = 0
      HTU(2) = 0
      HTU(3) = 0
      HTU(4) = HESS(8)*(U1(2)*U2(31))+HESS(9)*(U1(4)*U2(31))+HESS(184)*(U1(30)*U2(31))+HESS(221)*(U1(31)*U2(31))
      HTU(5) = HESS(10)*(U1(5)*U2(31))+HESS(185)*(U1(30)*U2(31))+HESS(222)*(U1(31)*U2(31))
      HTU(6) = 0
      HTU(7) = HESS(1)*(U1(1)*U2(35))+HESS(13)*(U1(7)*U2(35))+HESS(280)*(U1(35)*U2(35))
      HTU(8) = HESS(16)*(U1(8)*U2(31))+HESS(25)*(U1(13)*U2(31))+HESS(36)*(U1(16)*U2(31))+HESS(74)*(U1(22)*U2(31))+HESS(186) 
     &     *(U1(30)*U2(31))+HESS(223)*(U1(31)*U2(31))
      HTU(9) = 0
      HTU(10) = HESS(18)*(U1(10)*U2(31))+HESS(26)*(U1(13)*U2(31))+HESS(37)*(U1(16)*U2(31))+HESS(40)*(U1(17)*U2(31))+HESS(75) 
     &     *(U1(22)*U2(31))+HESS(106)*(U1(24)*U2(31))+HESS(187)*(U1(30)*U2(31))+HESS(224)*(U1(31)*U2(31))
      HTU(11) = HESS(19)*(U1(11)*U2(11))+HESS(20)*(U1(11)*U2(31))+HESS(225)*(U1(31)*U2(31))+HESS(281)*(U1(35)*U2(11)) 
     &     +HESS(282)*(U1(35)*U2(31))+HESS(303)*(U1(36)*U2(11))
      HTU(12) = HESS(23)*(U1(12)*U2(31))+HESS(226)*(U1(31)*U2(31))+HESS(283)*(U1(35)*U2(31))
      HTU(13) = HESS(2)*(U1(1)*U2(36))+HESS(27)*(U1(13)*U2(36))+HESS(62)*(U1(20)*U2(36))+HESS(188)*(U1(30)*U2(36))+HESS(284) 
     &     *(U1(35)*U2(36))+HESS(304)*(U1(36)*U2(36))
      HTU(14) = HESS(28)*(U1(14)*U2(31))+HESS(227)*(U1(31)*U2(31))+HESS(260)*(U1(33)*U2(31))
      HTU(15) = HESS(3)*(U1(1)*U2(35))+HESS(34)*(U1(15)*U2(35))+HESS(285)*(U1(35)*U2(35))
      HTU(16) = HESS(14)*(U1(7)*U2(31))+HESS(15)*(U1(7)*U2(33))+HESS(29)*(U1(14)*U2(33))+HESS(38)*(U1(16)*U2(31))+HESS(39) 
     &     *(U1(16)*U2(33))+HESS(63)*(U1(20)*U2(31))+HESS(76)*(U1(22)*U2(31))+HESS(189)*(U1(30)*U2(31))+HESS(228) 
     &     *(U1(31)*U2(31))+HESS(261)*(U1(33)*U2(33))
      HTU(17) = HESS(41)*(U1(17)*U2(31))+HESS(77)*(U1(22)*U2(31))+HESS(169)*(U1(29)*U2(31))+HESS(229)*(U1(31)*U2(31))
      HTU(18) = HESS(45)*(U1(18)*U2(31))+HESS(190)*(U1(30)*U2(31))+HESS(230)*(U1(31)*U2(31))
      HTU(19) = HESS(46)*(U1(18)*U2(32))+HESS(47)*(U1(18)*U2(34))+HESS(59)*(U1(19)*U2(31))+HESS(60)*(U1(19)*U2(32))+HESS(61) 
     &     *(U1(19)*U2(34))+HESS(78)*(U1(22)*U2(31))+HESS(79)*(U1(22)*U2(32))+HESS(119)*(U1(25)*U2(31))+HESS(120) 
     &     *(U1(25)*U2(32))+HESS(121)*(U1(25)*U2(34))+HESS(153)*(U1(28)*U2(31))+HESS(191)*(U1(30)*U2(31))+HESS(192) 
     &     *(U1(30)*U2(32))+HESS(193)*(U1(30)*U2(34))+HESS(231)*(U1(31)*U2(31))+HESS(232)*(U1(31)*U2(32))+HESS(253) 
     &     *(U1(32)*U2(32))+HESS(271)*(U1(34)*U2(34))
      HTU(20) = HESS(42)*(U1(17)*U2(34))+HESS(48)*(U1(18)*U2(31))+HESS(49)*(U1(18)*U2(34))+HESS(64)*(U1(20)*U2(31))+HESS(65) 
     &     *(U1(20)*U2(34))+HESS(80)*(U1(22)*U2(31))+HESS(81)*(U1(22)*U2(34))+HESS(122)*(U1(25)*U2(31))+HESS(123) 
     &     *(U1(25)*U2(34))+HESS(154)*(U1(28)*U2(34))+HESS(170)*(U1(29)*U2(31))+HESS(171)*(U1(29)*U2(34))+HESS(194) 
     &     *(U1(30)*U2(31))+HESS(195)*(U1(30)*U2(34))+HESS(233)*(U1(31)*U2(31))+HESS(234)*(U1(31)*U2(34))+HESS(272) 
     &     *(U1(34)*U2(34))
      HTU(21) = HESS(4)*(U1(1)*U2(36))+HESS(66)*(U1(21)*U2(21))+HESS(67)*(U1(21)*U2(22))+HESS(68)*(U1(21)*U2(30))+HESS(69) 
     &     *(U1(21)*U2(36))+HESS(82)*(U1(22)*U2(22))+HESS(196)*(U1(30)*U2(30))+HESS(305)*(U1(36)*U2(36))
      HTU(22) = HESS(67)*(U1(21)*U2(21))+HESS(82)*(U1(22)*U2(21))+HESS(83)*(U1(22)*U2(22))+HESS(84)*(U1(22)*U2(30))+HESS(85) 
     &     *(U1(22)*U2(36))+HESS(197)*(U1(30)*U2(30))+HESS(286)*(U1(35)*U2(36))+HESS(306)*(U1(36)*U2(36))
      HTU(23) = HESS(50)*(U1(18)*U2(32))+HESS(51)*(U1(18)*U2(34))+HESS(70)*(U1(21)*U2(32))+HESS(71)*(U1(21)*U2(33))+HESS(86) 
     &     *(U1(22)*U2(31))+HESS(87)*(U1(22)*U2(32))+HESS(88)*(U1(22)*U2(33))+HESS(89)*(U1(22)*U2(34))+HESS(102)*(U1(23) 
     &     *U2(31))+HESS(103)*(U1(23)*U2(32))+HESS(104)*(U1(23)*U2(33))+HESS(105)*(U1(23)*U2(34))+HESS(107)*(U1(24) 
     &     *U2(31))+HESS(108)*(U1(24)*U2(32))+HESS(109)*(U1(24)*U2(33))+HESS(110)*(U1(24)*U2(34))+HESS(124)*(U1(25) 
     &     *U2(31))+HESS(125)*(U1(25)*U2(32))+HESS(126)*(U1(25)*U2(33))+HESS(127)*(U1(25)*U2(34))+HESS(155)*(U1(28) 
     &     *U2(31))+HESS(156)*(U1(28)*U2(32))+HESS(157)*(U1(28)*U2(33))+HESS(158)*(U1(28)*U2(34))+HESS(198)*(U1(30) 
     &     *U2(31))+HESS(199)*(U1(30)*U2(32))+HESS(200)*(U1(30)*U2(34))+HESS(235)*(U1(31)*U2(31))+HESS(236)*(U1(31) 
     &     *U2(32))+HESS(237)*(U1(31)*U2(34))+HESS(254)*(U1(32)*U2(32))+HESS(262)*(U1(33)*U2(33))+HESS(273)*(U1(34) 
     &     *U2(34))+HESS(287)*(U1(35)*U2(33))
      HTU(24) = HESS(35)*(U1(15)*U2(31))+HESS(72)*(U1(21)*U2(31))+HESS(90)*(U1(22)*U2(31))+HESS(111)*(U1(24)*U2(31)) 
     &     +HESS(159)*(U1(28)*U2(31))+HESS(201)*(U1(30)*U2(31))+HESS(238)*(U1(31)*U2(31))
      HTU(25) = HESS(30)*(U1(14)*U2(33))+HESS(52)*(U1(18)*U2(31))+HESS(53)*(U1(18)*U2(32))+HESS(54)*(U1(18)*U2(33)) 
     &     +HESS(128)*(U1(25)*U2(31))+HESS(129)*(U1(25)*U2(32))+HESS(130)*(U1(25)*U2(33))+HESS(202)*(U1(30)*U2(31)) 
     &     +HESS(203)*(U1(30)*U2(32))+HESS(204)*(U1(30)*U2(33))+HESS(239)*(U1(31)*U2(31))+HESS(240)*(U1(31)*U2(32)) 
     &     +HESS(255)*(U1(32)*U2(32))+HESS(263)*(U1(33)*U2(33))
      HTU(26) = HESS(5)*(U1(1)*U2(33))+HESS(6)*(U1(1)*U2(35))+HESS(55)*(U1(18)*U2(34))+HESS(73)*(U1(21)*U2(31))+HESS(91) 
     &     *(U1(22)*U2(31))+HESS(92)*(U1(22)*U2(32))+HESS(93)*(U1(22)*U2(33))+HESS(94)*(U1(22)*U2(34))+HESS(95)*(U1(22) 
     &     *U2(35))+HESS(112)*(U1(24)*U2(32))+HESS(113)*(U1(24)*U2(33))+HESS(114)*(U1(24)*U2(34))+HESS(115)*(U1(24) 
     &     *U2(35))+HESS(131)*(U1(25)*U2(31))+HESS(132)*(U1(25)*U2(32))+HESS(133)*(U1(25)*U2(34))+HESS(140)*(U1(26) 
     &     *U2(31))+HESS(141)*(U1(26)*U2(32))+HESS(142)*(U1(26)*U2(33))+HESS(143)*(U1(26)*U2(34))+HESS(144)*(U1(26) 
     &     *U2(35))+HESS(145)*(U1(27)*U2(31))+HESS(146)*(U1(27)*U2(32))+HESS(147)*(U1(27)*U2(33))+HESS(148)*(U1(27) 
     &     *U2(34))+HESS(149)*(U1(27)*U2(35))+HESS(160)*(U1(28)*U2(33))+HESS(161)*(U1(28)*U2(34))+HESS(162)*(U1(28) 
     &     *U2(35))+HESS(172)*(U1(29)*U2(32))+HESS(173)*(U1(29)*U2(34))+HESS(205)*(U1(30)*U2(31))+HESS(206)*(U1(30) 
     &     *U2(32))+HESS(207)*(U1(30)*U2(33))+HESS(208)*(U1(30)*U2(34))+HESS(209)*(U1(30)*U2(35))+HESS(241)*(U1(31) 
     &     *U2(31))+HESS(242)*(U1(31)*U2(34))+HESS(256)*(U1(32)*U2(32))+HESS(264)*(U1(33)*U2(33))+HESS(274)*(U1(34) 
     &     *U2(34))+HESS(288)*(U1(35)*U2(33))+HESS(289)*(U1(35)*U2(35))+HESS(307)*(U1(36)*U2(35))
      HTU(27) = HESS(7)*(U1(1)*U2(33))+HESS(31)*(U1(14)*U2(33))+HESS(43)*(U1(17)*U2(31))+HESS(44)*(U1(17)*U2(34))+HESS(56) 
     &     *(U1(18)*U2(31))+HESS(57)*(U1(18)*U2(33))+HESS(58)*(U1(18)*U2(34))+HESS(96)*(U1(22)*U2(31))+HESS(97)*(U1(22) 
     &     *U2(33))+HESS(98)*(U1(22)*U2(34))+HESS(116)*(U1(24)*U2(31))+HESS(117)*(U1(24)*U2(33))+HESS(118)*(U1(24) 
     &     *U2(34))+HESS(134)*(U1(25)*U2(31))+HESS(135)*(U1(25)*U2(33))+HESS(136)*(U1(25)*U2(34))+HESS(150)*(U1(27) 
     &     *U2(31))+HESS(151)*(U1(27)*U2(33))+HESS(152)*(U1(27)*U2(34))+HESS(163)*(U1(28)*U2(31))+HESS(164)*(U1(28) 
     &     *U2(33))+HESS(165)*(U1(28)*U2(34))+HESS(174)*(U1(29)*U2(31))+HESS(175)*(U1(29)*U2(33))+HESS(176)*(U1(29) 
     &     *U2(34))+HESS(210)*(U1(30)*U2(31))+HESS(211)*(U1(30)*U2(33))+HESS(212)*(U1(30)*U2(34))+HESS(243)*(U1(31) 
     &     *U2(31))+HESS(244)*(U1(31)*U2(34))+HESS(265)*(U1(33)*U2(33))+HESS(275)*(U1(34)*U2(34))
      HTU(28) = HESS(32)*(U1(14)*U2(33))+HESS(166)*(U1(28)*U2(31))+HESS(167)*(U1(28)*U2(32))+HESS(168)*(U1(28)*U2(33)) 
     &     +HESS(177)*(U1(29)*U2(31))+HESS(178)*(U1(29)*U2(32))+HESS(179)*(U1(29)*U2(33))+HESS(245)*(U1(31)*U2(31)) 
     &     +HESS(246)*(U1(31)*U2(32))+HESS(257)*(U1(32)*U2(32))+HESS(266)*(U1(33)*U2(33))
      HTU(29) = HESS(12)*(U1(6)*U2(35))+HESS(99)*(U1(22)*U2(29))+HESS(100)*(U1(22)*U2(30))+HESS(101)*(U1(22)*U2(36)) 
     &     +HESS(137)*(U1(25)*U2(29))+HESS(138)*(U1(25)*U2(30))+HESS(139)*(U1(25)*U2(36))+HESS(180)*(U1(29)*U2(29)) 
     &     +HESS(181)*(U1(29)*U2(30))+HESS(182)*(U1(29)*U2(35))+HESS(183)*(U1(29)*U2(36))+HESS(213)*(U1(30)*U2(29)) 
     &     +HESS(214)*(U1(30)*U2(30))+HESS(215)*(U1(30)*U2(36))+HESS(247)*(U1(31)*U2(30))+HESS(290)*(U1(35)*U2(35)) 
     &     +HESS(291)*(U1(35)*U2(36))+HESS(308)*(U1(36)*U2(36))
      HTU(30) = HESS(11)*(U1(5)*U2(30))+HESS(24)*(U1(12)*U2(35))+HESS(68)*(U1(21)*U2(21))+HESS(84)*(U1(22)*U2(22))+HESS(100) 
     &     *(U1(22)*U2(29))+HESS(138)*(U1(25)*U2(29))+HESS(181)*(U1(29)*U2(29))+HESS(196)*(U1(30)*U2(21))+HESS(197) 
     &     *(U1(30)*U2(22))+HESS(214)*(U1(30)*U2(29))+HESS(216)*(U1(30)*U2(30))+HESS(217)*(U1(30)*U2(34))+HESS(218) 
     &     *(U1(30)*U2(35))+HESS(219)*(U1(30)*U2(36))+HESS(247)*(U1(31)*U2(29))+HESS(248)*(U1(31)*U2(34))+HESS(249) 
     &     *(U1(31)*U2(36))+HESS(276)*(U1(34)*U2(34))+HESS(292)*(U1(35)*U2(35))+HESS(293)*(U1(35)*U2(36))+HESS(309) 
     &     *(U1(36)*U2(36))
      HTU(31) = HESS(8)*(U1(2)*U2(4))+HESS(9)*(U1(4)*U2(4))+HESS(10)*(U1(5)*U2(5))+HESS(14)*(U1(7)*U2(16))+HESS(16)*(U1(8) 
     &     *U2(8))+HESS(18)*(U1(10)*U2(10))+HESS(20)*(U1(11)*U2(11))+HESS(21)*(U1(11)*U2(36))+HESS(23)*(U1(12)*U2(12)) 
     &     +HESS(25)*(U1(13)*U2(8))+HESS(26)*(U1(13)*U2(10))+HESS(28)*(U1(14)*U2(14))+HESS(33)*(U1(14)*U2(35))+HESS(35) 
     &     *(U1(15)*U2(24))+HESS(36)*(U1(16)*U2(8))+HESS(37)*(U1(16)*U2(10))+HESS(38)*(U1(16)*U2(16))+HESS(40)*(U1(17) 
     &     *U2(10))+HESS(41)*(U1(17)*U2(17))+HESS(43)*(U1(17)*U2(27))+HESS(45)*(U1(18)*U2(18))+HESS(48)*(U1(18)*U2(20)) 
     &     +HESS(52)*(U1(18)*U2(25))+HESS(56)*(U1(18)*U2(27))+HESS(59)*(U1(19)*U2(19))+HESS(63)*(U1(20)*U2(16))+HESS(64) 
     &     *(U1(20)*U2(20))+HESS(72)*(U1(21)*U2(24))+HESS(73)*(U1(21)*U2(26))+HESS(74)*(U1(22)*U2(8))+HESS(75)*(U1(22) 
     &     *U2(10))+HESS(76)*(U1(22)*U2(16))+HESS(77)*(U1(22)*U2(17))+HESS(78)*(U1(22)*U2(19))+HESS(80)*(U1(22)*U2(20)) 
     &     +HESS(86)*(U1(22)*U2(23))+HESS(90)*(U1(22)*U2(24))+HESS(91)*(U1(22)*U2(26))+HESS(96)*(U1(22)*U2(27)) 
     &     +HESS(102)*(U1(23)*U2(23))+HESS(106)*(U1(24)*U2(10))+HESS(107)*(U1(24)*U2(23))+HESS(111)*(U1(24)*U2(24)) 
     &     +HESS(116)*(U1(24)*U2(27))+HESS(119)*(U1(25)*U2(19))+HESS(122)*(U1(25)*U2(20))+HESS(124)*(U1(25)*U2(23)) 
     &     +HESS(128)*(U1(25)*U2(25))+HESS(131)*(U1(25)*U2(26))+HESS(134)*(U1(25)*U2(27))+HESS(140)*(U1(26)*U2(26)) 
     &     +HESS(145)*(U1(27)*U2(26))+HESS(150)*(U1(27)*U2(27))+HESS(153)*(U1(28)*U2(19))+HESS(155)*(U1(28)*U2(23)) 
     &     +HESS(159)*(U1(28)*U2(24))+HESS(163)*(U1(28)*U2(27))+HESS(166)*(U1(28)*U2(28))+HESS(169)*(U1(29)*U2(17)) 
     &     +HESS(170)*(U1(29)*U2(20))+HESS(174)*(U1(29)*U2(27))+HESS(177)*(U1(29)*U2(28))+HESS(184)*(U1(30)*U2(4)) 
     &     +HESS(185)*(U1(30)*U2(5))+HESS(186)*(U1(30)*U2(8))+HESS(187)*(U1(30)*U2(10))+HESS(189)*(U1(30)*U2(16)) 
     &     +HESS(190)*(U1(30)*U2(18))+HESS(191)*(U1(30)*U2(19))+HESS(194)*(U1(30)*U2(20))+HESS(198)*(U1(30)*U2(23)) 
     &     +HESS(201)*(U1(30)*U2(24))+HESS(202)*(U1(30)*U2(25))+HESS(205)*(U1(30)*U2(26))+HESS(210)*(U1(30)*U2(27)) 
     &     +HESS(220)*(U1(30)*U2(34))+HESS(221)*(U1(31)*U2(4))+HESS(222)*(U1(31)*U2(5))+HESS(223)*(U1(31)*U2(8)) 
     &     +HESS(224)*(U1(31)*U2(10))+HESS(225)*(U1(31)*U2(11))+HESS(226)*(U1(31)*U2(12))+HESS(227)*(U1(31)*U2(14)) 
     &     +HESS(228)*(U1(31)*U2(16))+HESS(229)*(U1(31)*U2(17))+HESS(230)*(U1(31)*U2(18))+HESS(231)*(U1(31)*U2(19)) 
     &     +HESS(233)*(U1(31)*U2(20))+HESS(235)*(U1(31)*U2(23))+HESS(238)*(U1(31)*U2(24))+HESS(239)*(U1(31)*U2(25)) 
     &     +HESS(241)*(U1(31)*U2(26))+HESS(243)*(U1(31)*U2(27))+HESS(245)*(U1(31)*U2(28))+HESS(250)*(U1(31)*U2(34)) 
     &     +HESS(251)*(U1(31)*U2(35))+HESS(252)*(U1(31)*U2(36))+HESS(260)*(U1(33)*U2(14))+HESS(277)*(U1(34)*U2(34)) 
     &     +HESS(282)*(U1(35)*U2(11))+HESS(283)*(U1(35)*U2(12))+HESS(294)*(U1(35)*U2(35))+HESS(310)*(U1(36)*U2(36))
      HTU(32) = HESS(46)*(U1(18)*U2(19))+HESS(50)*(U1(18)*U2(23))+HESS(53)*(U1(18)*U2(25))+HESS(60)*(U1(19)*U2(19))+HESS(70) 
     &     *(U1(21)*U2(23))+HESS(79)*(U1(22)*U2(19))+HESS(87)*(U1(22)*U2(23))+HESS(92)*(U1(22)*U2(26))+HESS(103)*(U1(23) 
     &     *U2(23))+HESS(108)*(U1(24)*U2(23))+HESS(112)*(U1(24)*U2(26))+HESS(120)*(U1(25)*U2(19))+HESS(125)*(U1(25) 
     &     *U2(23))+HESS(129)*(U1(25)*U2(25))+HESS(132)*(U1(25)*U2(26))+HESS(141)*(U1(26)*U2(26))+HESS(146)*(U1(27) 
     &     *U2(26))+HESS(156)*(U1(28)*U2(23))+HESS(167)*(U1(28)*U2(28))+HESS(172)*(U1(29)*U2(26))+HESS(178)*(U1(29) 
     &     *U2(28))+HESS(192)*(U1(30)*U2(19))+HESS(199)*(U1(30)*U2(23))+HESS(203)*(U1(30)*U2(25))+HESS(206)*(U1(30) 
     &     *U2(26))+HESS(232)*(U1(31)*U2(19))+HESS(236)*(U1(31)*U2(23))+HESS(240)*(U1(31)*U2(25))+HESS(246)*(U1(31) 
     &     *U2(28))+HESS(253)*(U1(32)*U2(19))+HESS(254)*(U1(32)*U2(23))+HESS(255)*(U1(32)*U2(25))+HESS(256)*(U1(32) 
     &     *U2(26))+HESS(257)*(U1(32)*U2(28))+HESS(258)*(U1(32)*U2(35))+HESS(259)*(U1(32)*U2(36))+HESS(267)*(U1(33) 
     &     *U2(35))+HESS(295)*(U1(35)*U2(35))+HESS(296)*(U1(35)*U2(36))+HESS(311)*(U1(36)*U2(35))+HESS(312)*(U1(36) 
     &     *U2(36))
      HTU(33) = HESS(5)*(U1(1)*U2(26))+HESS(7)*(U1(1)*U2(27))+HESS(15)*(U1(7)*U2(16))+HESS(17)*(U1(9)*U2(35))+HESS(29) 
     &     *(U1(14)*U2(16))+HESS(30)*(U1(14)*U2(25))+HESS(31)*(U1(14)*U2(27))+HESS(32)*(U1(14)*U2(28))+HESS(39)*(U1(16) 
     &     *U2(16))+HESS(54)*(U1(18)*U2(25))+HESS(57)*(U1(18)*U2(27))+HESS(71)*(U1(21)*U2(23))+HESS(88)*(U1(22)*U2(23)) 
     &     +HESS(93)*(U1(22)*U2(26))+HESS(97)*(U1(22)*U2(27))+HESS(104)*(U1(23)*U2(23))+HESS(109)*(U1(24)*U2(23)) 
     &     +HESS(113)*(U1(24)*U2(26))+HESS(117)*(U1(24)*U2(27))+HESS(126)*(U1(25)*U2(23))+HESS(130)*(U1(25)*U2(25)) 
     &     +HESS(135)*(U1(25)*U2(27))+HESS(142)*(U1(26)*U2(26))+HESS(147)*(U1(27)*U2(26))+HESS(151)*(U1(27)*U2(27)) 
     &     +HESS(157)*(U1(28)*U2(23))+HESS(160)*(U1(28)*U2(26))+HESS(164)*(U1(28)*U2(27))+HESS(168)*(U1(28)*U2(28)) 
     &     +HESS(175)*(U1(29)*U2(27))+HESS(179)*(U1(29)*U2(28))+HESS(204)*(U1(30)*U2(25))+HESS(207)*(U1(30)*U2(26)) 
     &     +HESS(211)*(U1(30)*U2(27))+HESS(261)*(U1(33)*U2(16))+HESS(262)*(U1(33)*U2(23))+HESS(263)*(U1(33)*U2(25)) 
     &     +HESS(264)*(U1(33)*U2(26))+HESS(265)*(U1(33)*U2(27))+HESS(266)*(U1(33)*U2(28))+HESS(268)*(U1(33)*U2(35)) 
     &     +HESS(269)*(U1(33)*U2(36))+HESS(287)*(U1(35)*U2(23))+HESS(288)*(U1(35)*U2(26))+HESS(297)*(U1(35)*U2(35)) 
     &     +HESS(298)*(U1(35)*U2(36))+HESS(313)*(U1(36)*U2(35))+HESS(314)*(U1(36)*U2(36))
      HTU(34) = HESS(42)*(U1(17)*U2(20))+HESS(44)*(U1(17)*U2(27))+HESS(47)*(U1(18)*U2(19))+HESS(49)*(U1(18)*U2(20))+HESS(51) 
     &     *(U1(18)*U2(23))+HESS(55)*(U1(18)*U2(26))+HESS(58)*(U1(18)*U2(27))+HESS(61)*(U1(19)*U2(19))+HESS(65)*(U1(20) 
     &     *U2(20))+HESS(81)*(U1(22)*U2(20))+HESS(89)*(U1(22)*U2(23))+HESS(94)*(U1(22)*U2(26))+HESS(98)*(U1(22)*U2(27)) 
     &     +HESS(105)*(U1(23)*U2(23))+HESS(110)*(U1(24)*U2(23))+HESS(114)*(U1(24)*U2(26))+HESS(118)*(U1(24)*U2(27)) 
     &     +HESS(121)*(U1(25)*U2(19))+HESS(123)*(U1(25)*U2(20))+HESS(127)*(U1(25)*U2(23))+HESS(133)*(U1(25)*U2(26)) 
     &     +HESS(136)*(U1(25)*U2(27))+HESS(143)*(U1(26)*U2(26))+HESS(148)*(U1(27)*U2(26))+HESS(152)*(U1(27)*U2(27)) 
     &     +HESS(154)*(U1(28)*U2(20))+HESS(158)*(U1(28)*U2(23))+HESS(161)*(U1(28)*U2(26))+HESS(165)*(U1(28)*U2(27)) 
     &     +HESS(171)*(U1(29)*U2(20))+HESS(173)*(U1(29)*U2(26))+HESS(176)*(U1(29)*U2(27))+HESS(193)*(U1(30)*U2(19)) 
     &     +HESS(195)*(U1(30)*U2(20))+HESS(200)*(U1(30)*U2(23))+HESS(208)*(U1(30)*U2(26))+HESS(212)*(U1(30)*U2(27)) 
     &     +HESS(217)*(U1(30)*U2(30))+HESS(220)*(U1(30)*U2(31))+HESS(234)*(U1(31)*U2(20))+HESS(237)*(U1(31)*U2(23)) 
     &     +HESS(242)*(U1(31)*U2(26))+HESS(244)*(U1(31)*U2(27))+HESS(248)*(U1(31)*U2(30))+HESS(250)*(U1(31)*U2(31)) 
     &     +HESS(270)*(U1(33)*U2(35))+HESS(271)*(U1(34)*U2(19))+HESS(272)*(U1(34)*U2(20))+HESS(273)*(U1(34)*U2(23)) 
     &     +HESS(274)*(U1(34)*U2(26))+HESS(275)*(U1(34)*U2(27))+HESS(276)*(U1(34)*U2(30))+HESS(277)*(U1(34)*U2(31)) 
     &     +HESS(278)*(U1(34)*U2(35))+HESS(279)*(U1(34)*U2(36))+HESS(299)*(U1(35)*U2(35))+HESS(300)*(U1(35)*U2(36)) 
     &     +HESS(315)*(U1(36)*U2(36))
      HTU(35) = HESS(1)*(U1(1)*U2(7))+HESS(3)*(U1(1)*U2(15))+HESS(6)*(U1(1)*U2(26))+HESS(12)*(U1(6)*U2(29))+HESS(13)*(U1(7) 
     &     *U2(7))+HESS(17)*(U1(9)*U2(33))+HESS(22)*(U1(11)*U2(36))+HESS(24)*(U1(12)*U2(30))+HESS(33)*(U1(14)*U2(31)) 
     &     +HESS(34)*(U1(15)*U2(15))+HESS(95)*(U1(22)*U2(26))+HESS(115)*(U1(24)*U2(26))+HESS(144)*(U1(26)*U2(26)) 
     &     +HESS(149)*(U1(27)*U2(26))+HESS(162)*(U1(28)*U2(26))+HESS(182)*(U1(29)*U2(29))+HESS(209)*(U1(30)*U2(26)) 
     &     +HESS(218)*(U1(30)*U2(30))+HESS(251)*(U1(31)*U2(31))+HESS(258)*(U1(32)*U2(32))+HESS(267)*(U1(33)*U2(32)) 
     &     +HESS(268)*(U1(33)*U2(33))+HESS(270)*(U1(33)*U2(34))+HESS(278)*(U1(34)*U2(34))+HESS(280)*(U1(35)*U2(7)) 
     &     +HESS(285)*(U1(35)*U2(15))+HESS(289)*(U1(35)*U2(26))+HESS(290)*(U1(35)*U2(29))+HESS(292)*(U1(35)*U2(30)) 
     &     +HESS(294)*(U1(35)*U2(31))+HESS(295)*(U1(35)*U2(32))+HESS(297)*(U1(35)*U2(33))+HESS(299)*(U1(35)*U2(34)) 
     &     +HESS(301)*(U1(35)*U2(36))+HESS(307)*(U1(36)*U2(26))+HESS(311)*(U1(36)*U2(32))+HESS(313)*(U1(36)*U2(33)) 
     &     +HESS(316)*(U1(36)*U2(36))
      HTU(36) = HESS(2)*(U1(1)*U2(13))+HESS(4)*(U1(1)*U2(21))+HESS(21)*(U1(11)*U2(31))+HESS(22)*(U1(11)*U2(35))+HESS(27) 
     &     *(U1(13)*U2(13))+HESS(62)*(U1(20)*U2(13))+HESS(69)*(U1(21)*U2(21))+HESS(85)*(U1(22)*U2(22))+HESS(101)*(U1(22) 
     &     *U2(29))+HESS(139)*(U1(25)*U2(29))+HESS(183)*(U1(29)*U2(29))+HESS(188)*(U1(30)*U2(13))+HESS(215)*(U1(30) 
     &     *U2(29))+HESS(219)*(U1(30)*U2(30))+HESS(249)*(U1(31)*U2(30))+HESS(252)*(U1(31)*U2(31))+HESS(259)*(U1(32) 
     &     *U2(32))+HESS(269)*(U1(33)*U2(33))+HESS(279)*(U1(34)*U2(34))+HESS(284)*(U1(35)*U2(13))+HESS(286)*(U1(35) 
     &     *U2(22))+HESS(291)*(U1(35)*U2(29))+HESS(293)*(U1(35)*U2(30))+HESS(296)*(U1(35)*U2(32))+HESS(298)*(U1(35) 
     &     *U2(33))+HESS(300)*(U1(35)*U2(34))+HESS(301)*(U1(35)*U2(35))+HESS(302)*(U1(35)*U2(36))+HESS(304)*(U1(36) 
     &     *U2(13))+HESS(305)*(U1(36)*U2(21))+HESS(306)*(U1(36)*U2(22))+HESS(308)*(U1(36)*U2(29))+HESS(309)*(U1(36) 
     &     *U2(30))+HESS(310)*(U1(36)*U2(31))+HESS(312)*(U1(36)*U2(32))+HESS(314)*(U1(36)*U2(33))+HESS(315)*(U1(36)
     &     *U2(34))+HESS(316)*(U1(36)*U2(35))+HESS(317)*(U1(36)*U2(36))
      
      END SUBROUTINE HessTR_Vec

C     End of HessTR_Vec function
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C     
C     Hess_Vec - Hessian times user vectors
C     Arguments :
C     HESS      - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
C     U1        - User vector
C     U2        - User vector
C     HU        - Hessian times user vectors: (Hess x U2) * U1 = [d (Jac*U1)/d Var] * U2
C     
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      SUBROUTINE Hess_Vec ( HESS, U1, U2, HU )

C     HESS - Hessian of Var (i.e. the 3-tensor d Jac / d Var)
      REAL(kind=dp) :: HESS(NHESS)
C     U1 - User vector
      REAL(kind=dp) :: U1(KPPNSP)
C     U2 - User vector
      REAL(kind=dp) :: U2(KPPNSP)
C     HU - Hessian times user vectors: (Hess x U2) * U1 = [d (Jac*U1)/d Var] * U2
      REAL(kind=dp) :: HU(KPPNSP)

C     Compute the vector HU =(Hess x U2) * U1 = d (Jac*U1)/d Var * U2
      HU(1) = HESS(1)*(U1(7)*U2(35))+HESS(1)*(U1(35)*U2(7))+HESS(2)*(U1(13)*U2(36))+HESS(2)*(U1(36)*U2(13))+HESS(3)*(U1(15) 
     &     *U2(35))+HESS(3)*(U1(35)*U2(15))+HESS(4)*(U1(21)*U2(36))+HESS(4)*(U1(36)*U2(21))+HESS(5)*(U1(26)*U2(33)) 
     &     +HESS(5)*(U1(33)*U2(26))+HESS(6)*(U1(26)*U2(35))+HESS(6)*(U1(35)*U2(26))+HESS(7)*(U1(27)*U2(33))+HESS(7) 
     &     *(U1(33)*U2(27))
      HU(2) = HESS(8)*(U1(4)*U2(31))+HESS(8)*(U1(31)*U2(4))
      HU(3) = 0
      HU(4) = HESS(9)*(U1(4)*U2(31))+HESS(9)*(U1(31)*U2(4))
      HU(5) = HESS(10)*(U1(5)*U2(31))+HESS(10)*(U1(31)*U2(5))+HESS(11)*(U1(30)*U2(30))
      HU(6) = HESS(12)*(U1(29)*U2(35))+HESS(12)*(U1(35)*U2(29))
      HU(7) = HESS(13)*(U1(7)*U2(35))+HESS(13)*(U1(35)*U2(7))+HESS(14)*(U1(16)*U2(31))+HESS(14)*(U1(31)*U2(16))+HESS(15) 
     &     *(U1(16)*U2(33))+HESS(15)*(U1(33)*U2(16))
      HU(8) = HESS(16)*(U1(8)*U2(31))+HESS(16)*(U1(31)*U2(8))
      HU(9) = HESS(17)*(U1(33)*U2(35))+HESS(17)*(U1(35)*U2(33))
      HU(10) = HESS(18)*(U1(10)*U2(31))+HESS(18)*(U1(31)*U2(10))
      HU(11) = HESS(19)*(U1(11)*U2(11))+HESS(20)*(U1(11)*U2(31))+HESS(20)*(U1(31)*U2(11))+HESS(21)*(U1(31)*U2(36))+HESS(21) 
     &     *(U1(36)*U2(31))+HESS(22)*(U1(35)*U2(36))+HESS(22)*(U1(36)*U2(35))
      HU(12) = HESS(23)*(U1(12)*U2(31))+HESS(23)*(U1(31)*U2(12))+HESS(24)*(U1(30)*U2(35))+HESS(24)*(U1(35)*U2(30))
      HU(13) = HESS(25)*(U1(8)*U2(31))+HESS(25)*(U1(31)*U2(8))+HESS(26)*(U1(10)*U2(31))+HESS(26)*(U1(31)*U2(10))+HESS(27) 
     &     *(U1(13)*U2(36))+HESS(27)*(U1(36)*U2(13))
      HU(14) = HESS(28)*(U1(14)*U2(31))+HESS(28)*(U1(31)*U2(14))+HESS(29)*(U1(16)*U2(33))+HESS(29)*(U1(33)*U2(16))+HESS(30) 
     &     *(U1(25)*U2(33))+HESS(30)*(U1(33)*U2(25))+HESS(31)*(U1(27)*U2(33))+HESS(31)*(U1(33)*U2(27))+HESS(32)*(U1(28) 
     &     *U2(33))+HESS(32)*(U1(33)*U2(28))+HESS(33)*(U1(31)*U2(35))+HESS(33)*(U1(35)*U2(31))
      HU(15) = HESS(34)*(U1(15)*U2(35))+HESS(34)*(U1(35)*U2(15))+HESS(35)*(U1(24)*U2(31))+HESS(35)*(U1(31)*U2(24))
      HU(16) = HESS(36)*(U1(8)*U2(31))+HESS(36)*(U1(31)*U2(8))+HESS(37)*(U1(10)*U2(31))+HESS(37)*(U1(31)*U2(10))+HESS(38) 
     &     *(U1(16)*U2(31))+HESS(38)*(U1(31)*U2(16))+HESS(39)*(U1(16)*U2(33))+HESS(39)*(U1(33)*U2(16))
      HU(17) = HESS(40)*(U1(10)*U2(31))+HESS(40)*(U1(31)*U2(10))+HESS(41)*(U1(17)*U2(31))+HESS(41)*(U1(31)*U2(17))+HESS(42) 
     &     *(U1(20)*U2(34))+HESS(42)*(U1(34)*U2(20))+HESS(43)*(U1(27)*U2(31))+HESS(43)*(U1(31)*U2(27))+HESS(44)*(U1(27) 
     &     *U2(34))+HESS(44)*(U1(34)*U2(27))
      HU(18) = HESS(45)*(U1(18)*U2(31))+HESS(45)*(U1(31)*U2(18))+HESS(46)*(U1(19)*U2(32))+HESS(46)*(U1(32)*U2(19))+HESS(47) 
     &     *(U1(19)*U2(34))+HESS(47)*(U1(34)*U2(19))+HESS(48)*(U1(20)*U2(31))+HESS(48)*(U1(31)*U2(20))+HESS(49)*(U1(20) 
     &     *U2(34))+HESS(49)*(U1(34)*U2(20))+HESS(50)*(U1(23)*U2(32))+HESS(50)*(U1(32)*U2(23))+HESS(51)*(U1(23)*U2(34)) 
     &     +HESS(51)*(U1(34)*U2(23))+HESS(52)*(U1(25)*U2(31))+HESS(52)*(U1(31)*U2(25))+HESS(53)*(U1(25)*U2(32))+HESS(53) 
     &     *(U1(32)*U2(25))+HESS(54)*(U1(25)*U2(33))+HESS(54)*(U1(33)*U2(25))+HESS(55)*(U1(26)*U2(34))+HESS(55)*(U1(34) 
     &     *U2(26))+HESS(56)*(U1(27)*U2(31))+HESS(56)*(U1(31)*U2(27))+HESS(57)*(U1(27)*U2(33))+HESS(57)*(U1(33)*U2(27)) 
     &     +HESS(58)*(U1(27)*U2(34))+HESS(58)*(U1(34)*U2(27))
      HU(19) = HESS(59)*(U1(19)*U2(31))+HESS(59)*(U1(31)*U2(19))+HESS(60)*(U1(19)*U2(32))+HESS(60)*(U1(32)*U2(19))+HESS(61) 
     &     *(U1(19)*U2(34))+HESS(61)*(U1(34)*U2(19))
      HU(20) = HESS(62)*(U1(13)*U2(36))+HESS(62)*(U1(36)*U2(13))+HESS(63)*(U1(16)*U2(31))+HESS(63)*(U1(31)*U2(16))+HESS(64) 
     &     *(U1(20)*U2(31))+HESS(64)*(U1(31)*U2(20))+HESS(65)*(U1(20)*U2(34))+HESS(65)*(U1(34)*U2(20))
      HU(21) = HESS(66)*(U1(21)*U2(21))+HESS(67)*(U1(21)*U2(22))+HESS(67)*(U1(22)*U2(21))+HESS(68)*(U1(21)*U2(30))+HESS(68) 
     &     *(U1(30)*U2(21))+HESS(69)*(U1(21)*U2(36))+HESS(69)*(U1(36)*U2(21))+HESS(70)*(U1(23)*U2(32))+HESS(70)*(U1(32) 
     &     *U2(23))+HESS(71)*(U1(23)*U2(33))+HESS(71)*(U1(33)*U2(23))+HESS(72)*(U1(24)*U2(31))+HESS(72)*(U1(31)*U2(24)) 
     &     +HESS(73)*(U1(26)*U2(31))+HESS(73)*(U1(31)*U2(26))
      HU(22) = HESS(74)*(U1(8)*U2(31))+HESS(74)*(U1(31)*U2(8))+HESS(75)*(U1(10)*U2(31))+HESS(75)*(U1(31)*U2(10))+HESS(76) 
     &     *(U1(16)*U2(31))+HESS(76)*(U1(31)*U2(16))+HESS(77)*(U1(17)*U2(31))+HESS(77)*(U1(31)*U2(17))+HESS(78)*(U1(19) 
     &     *U2(31))+HESS(78)*(U1(31)*U2(19))+HESS(79)*(U1(19)*U2(32))+HESS(79)*(U1(32)*U2(19))+HESS(80)*(U1(20)*U2(31)) 
     &     +HESS(80)*(U1(31)*U2(20))+HESS(81)*(U1(20)*U2(34))+HESS(81)*(U1(34)*U2(20))+HESS(82)*(U1(21)*U2(22))+HESS(82) 
     &     *(U1(22)*U2(21))+HESS(83)*(U1(22)*U2(22))+HESS(84)*(U1(22)*U2(30))+HESS(84)*(U1(30)*U2(22))+HESS(85)*(U1(22) 
     &     *U2(36))+HESS(85)*(U1(36)*U2(22))+HESS(86)*(U1(23)*U2(31))+HESS(86)*(U1(31)*U2(23))+HESS(87)*(U1(23)*U2(32)) 
     &     +HESS(87)*(U1(32)*U2(23))+HESS(88)*(U1(23)*U2(33))+HESS(88)*(U1(33)*U2(23))+HESS(89)*(U1(23)*U2(34))+HESS(89) 
     &     *(U1(34)*U2(23))+HESS(90)*(U1(24)*U2(31))+HESS(90)*(U1(31)*U2(24))+HESS(91)*(U1(26)*U2(31))+HESS(91)*(U1(31) 
     &     *U2(26))+HESS(92)*(U1(26)*U2(32))+HESS(92)*(U1(32)*U2(26))+HESS(93)*(U1(26)*U2(33))+HESS(93)*(U1(33)*U2(26)) 
     &     +HESS(94)*(U1(26)*U2(34))+HESS(94)*(U1(34)*U2(26))+HESS(95)*(U1(26)*U2(35))+HESS(95)*(U1(35)*U2(26))+HESS(96) 
     &     *(U1(27)*U2(31))+HESS(96)*(U1(31)*U2(27))+HESS(97)*(U1(27)*U2(33))+HESS(97)*(U1(33)*U2(27))+HESS(98)*(U1(27) 
     &     *U2(34))+HESS(98)*(U1(34)*U2(27))+HESS(99)*(U1(29)*U2(29))+HESS(100)*(U1(29)*U2(30))+HESS(100)*(U1(30)*U2(29)) 
     &     +HESS(101)*(U1(29)*U2(36))+HESS(101)*(U1(36)*U2(29))
      HU(23) = HESS(102)*(U1(23)*U2(31))+HESS(102)*(U1(31)*U2(23))+HESS(103)*(U1(23)*U2(32))+HESS(103)*(U1(32)*U2(23)) 
     &     +HESS(104)*(U1(23)*U2(33))+HESS(104)*(U1(33)*U2(23))+HESS(105)*(U1(23)*U2(34))+HESS(105)*(U1(34)*U2(23))
      HU(24) = HESS(106)*(U1(10)*U2(31))+HESS(106)*(U1(31)*U2(10))+HESS(107)*(U1(23)*U2(31))+HESS(107)*(U1(31)*U2(23)) 
     &     +HESS(108)*(U1(23)*U2(32))+HESS(108)*(U1(32)*U2(23))+HESS(109)*(U1(23)*U2(33))+HESS(109)*(U1(33)*U2(23)) 
     &     +HESS(110)*(U1(23)*U2(34))+HESS(110)*(U1(34)*U2(23))+HESS(111)*(U1(24)*U2(31))+HESS(111)*(U1(31)*U2(24)) 
     &     +HESS(112)*(U1(26)*U2(32))+HESS(112)*(U1(32)*U2(26))+HESS(113)*(U1(26)*U2(33))+HESS(113)*(U1(33)*U2(26)) 
     &     +HESS(114)*(U1(26)*U2(34))+HESS(114)*(U1(34)*U2(26))+HESS(115)*(U1(26)*U2(35))+HESS(115)*(U1(35)*U2(26)) 
     &     +HESS(116)*(U1(27)*U2(31))+HESS(116)*(U1(31)*U2(27))+HESS(117)*(U1(27)*U2(33))+HESS(117)*(U1(33)*U2(27)) 
     &     +HESS(118)*(U1(27)*U2(34))+HESS(118)*(U1(34)*U2(27))
      HU(25) = HESS(119)*(U1(19)*U2(31))+HESS(119)*(U1(31)*U2(19))+HESS(120)*(U1(19)*U2(32))+HESS(120)*(U1(32)*U2(19)) 
     &     +HESS(121)*(U1(19)*U2(34))+HESS(121)*(U1(34)*U2(19))+HESS(122)*(U1(20)*U2(31))+HESS(122)*(U1(31)*U2(20)) 
     &     +HESS(123)*(U1(20)*U2(34))+HESS(123)*(U1(34)*U2(20))+HESS(124)*(U1(23)*U2(31))+HESS(124)*(U1(31)*U2(23)) 
     &     +HESS(125)*(U1(23)*U2(32))+HESS(125)*(U1(32)*U2(23))+HESS(126)*(U1(23)*U2(33))+HESS(126)*(U1(33)*U2(23)) 
     &     +HESS(127)*(U1(23)*U2(34))+HESS(127)*(U1(34)*U2(23))+HESS(128)*(U1(25)*U2(31))+HESS(128)*(U1(31)*U2(25)) 
     &     +HESS(129)*(U1(25)*U2(32))+HESS(129)*(U1(32)*U2(25))+HESS(130)*(U1(25)*U2(33))+HESS(130)*(U1(33)*U2(25)) 
     &     +HESS(131)*(U1(26)*U2(31))+HESS(131)*(U1(31)*U2(26))+HESS(132)*(U1(26)*U2(32))+HESS(132)*(U1(32)*U2(26)) 
     &     +HESS(133)*(U1(26)*U2(34))+HESS(133)*(U1(34)*U2(26))+HESS(134)*(U1(27)*U2(31))+HESS(134)*(U1(31)*U2(27)) 
     &     +HESS(135)*(U1(27)*U2(33))+HESS(135)*(U1(33)*U2(27))+HESS(136)*(U1(27)*U2(34))+HESS(136)*(U1(34)*U2(27)) 
     &     +HESS(137)*(U1(29)*U2(29))+HESS(138)*(U1(29)*U2(30))+HESS(138)*(U1(30)*U2(29))+HESS(139)*(U1(29)*U2(36)) 
     &     +HESS(139)*(U1(36)*U2(29))
      HU(26) = HESS(140)*(U1(26)*U2(31))+HESS(140)*(U1(31)*U2(26))+HESS(141)*(U1(26)*U2(32))+HESS(141)*(U1(32)*U2(26)) 
     &     +HESS(142)*(U1(26)*U2(33))+HESS(142)*(U1(33)*U2(26))+HESS(143)*(U1(26)*U2(34))+HESS(143)*(U1(34)*U2(26)) 
     &     +HESS(144)*(U1(26)*U2(35))+HESS(144)*(U1(35)*U2(26))
      HU(27) = HESS(145)*(U1(26)*U2(31))+HESS(145)*(U1(31)*U2(26))+HESS(146)*(U1(26)*U2(32))+HESS(146)*(U1(32)*U2(26)) 
     &     +HESS(147)*(U1(26)*U2(33))+HESS(147)*(U1(33)*U2(26))+HESS(148)*(U1(26)*U2(34))+HESS(148)*(U1(34)*U2(26)) 
     &     +HESS(149)*(U1(26)*U2(35))+HESS(149)*(U1(35)*U2(26))+HESS(150)*(U1(27)*U2(31))+HESS(150)*(U1(31)*U2(27)) 
     &     +HESS(151)*(U1(27)*U2(33))+HESS(151)*(U1(33)*U2(27))+HESS(152)*(U1(27)*U2(34))+HESS(152)*(U1(34)*U2(27))
      HU(28) = HESS(153)*(U1(19)*U2(31))+HESS(153)*(U1(31)*U2(19))+HESS(154)*(U1(20)*U2(34))+HESS(154)*(U1(34)*U2(20)) 
     &     +HESS(155)*(U1(23)*U2(31))+HESS(155)*(U1(31)*U2(23))+HESS(156)*(U1(23)*U2(32))+HESS(156)*(U1(32)*U2(23)) 
     &     +HESS(157)*(U1(23)*U2(33))+HESS(157)*(U1(33)*U2(23))+HESS(158)*(U1(23)*U2(34))+HESS(158)*(U1(34)*U2(23)) 
     &     +HESS(159)*(U1(24)*U2(31))+HESS(159)*(U1(31)*U2(24))+HESS(160)*(U1(26)*U2(33))+HESS(160)*(U1(33)*U2(26)) 
     &     +HESS(161)*(U1(26)*U2(34))+HESS(161)*(U1(34)*U2(26))+HESS(162)*(U1(26)*U2(35))+HESS(162)*(U1(35)*U2(26)) 
     &     +HESS(163)*(U1(27)*U2(31))+HESS(163)*(U1(31)*U2(27))+HESS(164)*(U1(27)*U2(33))+HESS(164)*(U1(33)*U2(27)) 
     &     +HESS(165)*(U1(27)*U2(34))+HESS(165)*(U1(34)*U2(27))+HESS(166)*(U1(28)*U2(31))+HESS(166)*(U1(31)*U2(28)) 
     &     +HESS(167)*(U1(28)*U2(32))+HESS(167)*(U1(32)*U2(28))+HESS(168)*(U1(28)*U2(33))+HESS(168)*(U1(33)*U2(28))
      HU(29) = HESS(169)*(U1(17)*U2(31))+HESS(169)*(U1(31)*U2(17))+HESS(170)*(U1(20)*U2(31))+HESS(170)*(U1(31)*U2(20)) 
     &     +HESS(171)*(U1(20)*U2(34))+HESS(171)*(U1(34)*U2(20))+HESS(172)*(U1(26)*U2(32))+HESS(172)*(U1(32)*U2(26)) 
     &     +HESS(173)*(U1(26)*U2(34))+HESS(173)*(U1(34)*U2(26))+HESS(174)*(U1(27)*U2(31))+HESS(174)*(U1(31)*U2(27)) 
     &     +HESS(175)*(U1(27)*U2(33))+HESS(175)*(U1(33)*U2(27))+HESS(176)*(U1(27)*U2(34))+HESS(176)*(U1(34)*U2(27)) 
     &     +HESS(177)*(U1(28)*U2(31))+HESS(177)*(U1(31)*U2(28))+HESS(178)*(U1(28)*U2(32))+HESS(178)*(U1(32)*U2(28)) 
     &     +HESS(179)*(U1(28)*U2(33))+HESS(179)*(U1(33)*U2(28))+HESS(180)*(U1(29)*U2(29))+HESS(181)*(U1(29)*U2(30)) 
     &     +HESS(181)*(U1(30)*U2(29))+HESS(182)*(U1(29)*U2(35))+HESS(182)*(U1(35)*U2(29))+HESS(183)*(U1(29)*U2(36)) 
     &     +HESS(183)*(U1(36)*U2(29))
      HU(30) = HESS(184)*(U1(4)*U2(31))+HESS(184)*(U1(31)*U2(4))+HESS(185)*(U1(5)*U2(31))+HESS(185)*(U1(31)*U2(5))+HESS(186) 
     &     *(U1(8)*U2(31))+HESS(186)*(U1(31)*U2(8))+HESS(187)*(U1(10)*U2(31))+HESS(187)*(U1(31)*U2(10))+HESS(188)*(U1(13) 
     &     *U2(36))+HESS(188)*(U1(36)*U2(13))+HESS(189)*(U1(16)*U2(31))+HESS(189)*(U1(31)*U2(16))+HESS(190)*(U1(18) 
     &     *U2(31))+HESS(190)*(U1(31)*U2(18))+HESS(191)*(U1(19)*U2(31))+HESS(191)*(U1(31)*U2(19))+HESS(192)*(U1(19) 
     &     *U2(32))+HESS(192)*(U1(32)*U2(19))+HESS(193)*(U1(19)*U2(34))+HESS(193)*(U1(34)*U2(19))+HESS(194)*(U1(20) 
     &     *U2(31))+HESS(194)*(U1(31)*U2(20))+HESS(195)*(U1(20)*U2(34))+HESS(195)*(U1(34)*U2(20))+HESS(196)*(U1(21) 
     &     *U2(30))+HESS(196)*(U1(30)*U2(21))+HESS(197)*(U1(22)*U2(30))+HESS(197)*(U1(30)*U2(22))+HESS(198)*(U1(23) 
     &     *U2(31))+HESS(198)*(U1(31)*U2(23))+HESS(199)*(U1(23)*U2(32))+HESS(199)*(U1(32)*U2(23))+HESS(200)*(U1(23) 
     &     *U2(34))+HESS(200)*(U1(34)*U2(23))+HESS(201)*(U1(24)*U2(31))+HESS(201)*(U1(31)*U2(24))+HESS(202)*(U1(25) 
     &     *U2(31))+HESS(202)*(U1(31)*U2(25))+HESS(203)*(U1(25)*U2(32))+HESS(203)*(U1(32)*U2(25))+HESS(204)*(U1(25) 
     &     *U2(33))+HESS(204)*(U1(33)*U2(25))+HESS(205)*(U1(26)*U2(31))+HESS(205)*(U1(31)*U2(26))+HESS(206)*(U1(26) 
     &     *U2(32))+HESS(206)*(U1(32)*U2(26))+HESS(207)*(U1(26)*U2(33))+HESS(207)*(U1(33)*U2(26))+HESS(208)*(U1(26) 
     &     *U2(34))+HESS(208)*(U1(34)*U2(26))+HESS(209)*(U1(26)*U2(35))+HESS(209)*(U1(35)*U2(26))+HESS(210)*(U1(27) 
     &     *U2(31))+HESS(210)*(U1(31)*U2(27))+HESS(211)*(U1(27)*U2(33))+HESS(211)*(U1(33)*U2(27))+HESS(212)*(U1(27) 
     &     *U2(34))+HESS(212)*(U1(34)*U2(27))+HESS(213)*(U1(29)*U2(29))+HESS(214)*(U1(29)*U2(30))+HESS(214)*(U1(30) 
     &     *U2(29))+HESS(215)*(U1(29)*U2(36))+HESS(215)*(U1(36)*U2(29))+HESS(216)*(U1(30)*U2(30))+HESS(217)*(U1(30) 
     &     *U2(34))+HESS(217)*(U1(34)*U2(30))+HESS(218)*(U1(30)*U2(35))+HESS(218)*(U1(35)*U2(30))+HESS(219)*(U1(30) 
     &     *U2(36))+HESS(219)*(U1(36)*U2(30))+HESS(220)*(U1(31)*U2(34))+HESS(220)*(U1(34)*U2(31))
      HU(31) = HESS(221)*(U1(4)*U2(31))+HESS(221)*(U1(31)*U2(4))+HESS(222)*(U1(5)*U2(31))+HESS(222)*(U1(31)*U2(5))+HESS(223) 
     &     *(U1(8)*U2(31))+HESS(223)*(U1(31)*U2(8))+HESS(224)*(U1(10)*U2(31))+HESS(224)*(U1(31)*U2(10))+HESS(225)*(U1(11) 
     &     *U2(31))+HESS(225)*(U1(31)*U2(11))+HESS(226)*(U1(12)*U2(31))+HESS(226)*(U1(31)*U2(12))+HESS(227)*(U1(14) 
     &     *U2(31))+HESS(227)*(U1(31)*U2(14))+HESS(228)*(U1(16)*U2(31))+HESS(228)*(U1(31)*U2(16))+HESS(229)*(U1(17) 
     &     *U2(31))+HESS(229)*(U1(31)*U2(17))+HESS(230)*(U1(18)*U2(31))+HESS(230)*(U1(31)*U2(18))+HESS(231)*(U1(19) 
     &     *U2(31))+HESS(231)*(U1(31)*U2(19))+HESS(232)*(U1(19)*U2(32))+HESS(232)*(U1(32)*U2(19))+HESS(233)*(U1(20) 
     &     *U2(31))+HESS(233)*(U1(31)*U2(20))+HESS(234)*(U1(20)*U2(34))+HESS(234)*(U1(34)*U2(20))+HESS(235)*(U1(23) 
     &     *U2(31))+HESS(235)*(U1(31)*U2(23))+HESS(236)*(U1(23)*U2(32))+HESS(236)*(U1(32)*U2(23))+HESS(237)*(U1(23) 
     &     *U2(34))+HESS(237)*(U1(34)*U2(23))+HESS(238)*(U1(24)*U2(31))+HESS(238)*(U1(31)*U2(24))+HESS(239)*(U1(25) 
     &     *U2(31))+HESS(239)*(U1(31)*U2(25))+HESS(240)*(U1(25)*U2(32))+HESS(240)*(U1(32)*U2(25))+HESS(241)*(U1(26) 
     &     *U2(31))+HESS(241)*(U1(31)*U2(26))+HESS(242)*(U1(26)*U2(34))+HESS(242)*(U1(34)*U2(26))+HESS(243)*(U1(27) 
     &     *U2(31))+HESS(243)*(U1(31)*U2(27))+HESS(244)*(U1(27)*U2(34))+HESS(244)*(U1(34)*U2(27))+HESS(245)*(U1(28) 
     &     *U2(31))+HESS(245)*(U1(31)*U2(28))+HESS(246)*(U1(28)*U2(32))+HESS(246)*(U1(32)*U2(28))+HESS(247)*(U1(29) 
     &     *U2(30))+HESS(247)*(U1(30)*U2(29))+HESS(248)*(U1(30)*U2(34))+HESS(248)*(U1(34)*U2(30))+HESS(249)*(U1(30) 
     &     *U2(36))+HESS(249)*(U1(36)*U2(30))+HESS(250)*(U1(31)*U2(34))+HESS(250)*(U1(34)*U2(31))+HESS(251)*(U1(31) 
     &     *U2(35))+HESS(251)*(U1(35)*U2(31))+HESS(252)*(U1(31)*U2(36))+HESS(252)*(U1(36)*U2(31))
      HU(32) = HESS(253)*(U1(19)*U2(32))+HESS(253)*(U1(32)*U2(19))+HESS(254)*(U1(23)*U2(32))+HESS(254)*(U1(32)*U2(23)) 
     &     +HESS(255)*(U1(25)*U2(32))+HESS(255)*(U1(32)*U2(25))+HESS(256)*(U1(26)*U2(32))+HESS(256)*(U1(32)*U2(26)) 
     &     +HESS(257)*(U1(28)*U2(32))+HESS(257)*(U1(32)*U2(28))+HESS(258)*(U1(32)*U2(35))+HESS(258)*(U1(35)*U2(32)) 
     &     +HESS(259)*(U1(32)*U2(36))+HESS(259)*(U1(36)*U2(32))
      HU(33) = HESS(260)*(U1(14)*U2(31))+HESS(260)*(U1(31)*U2(14))+HESS(261)*(U1(16)*U2(33))+HESS(261)*(U1(33)*U2(16)) 
     &     +HESS(262)*(U1(23)*U2(33))+HESS(262)*(U1(33)*U2(23))+HESS(263)*(U1(25)*U2(33))+HESS(263)*(U1(33)*U2(25)) 
     &     +HESS(264)*(U1(26)*U2(33))+HESS(264)*(U1(33)*U2(26))+HESS(265)*(U1(27)*U2(33))+HESS(265)*(U1(33)*U2(27)) 
     &     +HESS(266)*(U1(28)*U2(33))+HESS(266)*(U1(33)*U2(28))+HESS(267)*(U1(32)*U2(35))+HESS(267)*(U1(35)*U2(32)) 
     &     +HESS(268)*(U1(33)*U2(35))+HESS(268)*(U1(35)*U2(33))+HESS(269)*(U1(33)*U2(36))+HESS(269)*(U1(36)*U2(33)) 
     &     +HESS(270)*(U1(34)*U2(35))+HESS(270)*(U1(35)*U2(34))
      HU(34) = HESS(271)*(U1(19)*U2(34))+HESS(271)*(U1(34)*U2(19))+HESS(272)*(U1(20)*U2(34))+HESS(272)*(U1(34)*U2(20)) 
     &     +HESS(273)*(U1(23)*U2(34))+HESS(273)*(U1(34)*U2(23))+HESS(274)*(U1(26)*U2(34))+HESS(274)*(U1(34)*U2(26)) 
     &     +HESS(275)*(U1(27)*U2(34))+HESS(275)*(U1(34)*U2(27))+HESS(276)*(U1(30)*U2(34))+HESS(276)*(U1(34)*U2(30)) 
     &     +HESS(277)*(U1(31)*U2(34))+HESS(277)*(U1(34)*U2(31))+HESS(278)*(U1(34)*U2(35))+HESS(278)*(U1(35)*U2(34)) 
     &     +HESS(279)*(U1(34)*U2(36))+HESS(279)*(U1(36)*U2(34))
      HU(35) = HESS(280)*(U1(7)*U2(35))+HESS(280)*(U1(35)*U2(7))+HESS(281)*(U1(11)*U2(11))+HESS(282)*(U1(11)*U2(31)) 
     &     +HESS(282)*(U1(31)*U2(11))+HESS(283)*(U1(12)*U2(31))+HESS(283)*(U1(31)*U2(12))+HESS(284)*(U1(13)*U2(36)) 
     &     +HESS(284)*(U1(36)*U2(13))+HESS(285)*(U1(15)*U2(35))+HESS(285)*(U1(35)*U2(15))+HESS(286)*(U1(22)*U2(36)) 
     &     +HESS(286)*(U1(36)*U2(22))+HESS(287)*(U1(23)*U2(33))+HESS(287)*(U1(33)*U2(23))+HESS(288)*(U1(26)*U2(33)) 
     &     +HESS(288)*(U1(33)*U2(26))+HESS(289)*(U1(26)*U2(35))+HESS(289)*(U1(35)*U2(26))+HESS(290)*(U1(29)*U2(35)) 
     &     +HESS(290)*(U1(35)*U2(29))+HESS(291)*(U1(29)*U2(36))+HESS(291)*(U1(36)*U2(29))+HESS(292)*(U1(30)*U2(35)) 
     &     +HESS(292)*(U1(35)*U2(30))+HESS(293)*(U1(30)*U2(36))+HESS(293)*(U1(36)*U2(30))+HESS(294)*(U1(31)*U2(35)) 
     &     +HESS(294)*(U1(35)*U2(31))+HESS(295)*(U1(32)*U2(35))+HESS(295)*(U1(35)*U2(32))+HESS(296)*(U1(32)*U2(36)) 
     &     +HESS(296)*(U1(36)*U2(32))+HESS(297)*(U1(33)*U2(35))+HESS(297)*(U1(35)*U2(33))+HESS(298)*(U1(33)*U2(36)) 
     &     +HESS(298)*(U1(36)*U2(33))+HESS(299)*(U1(34)*U2(35))+HESS(299)*(U1(35)*U2(34))+HESS(300)*(U1(34)*U2(36)) 
     &     +HESS(300)*(U1(36)*U2(34))+HESS(301)*(U1(35)*U2(36))+HESS(301)*(U1(36)*U2(35))+HESS(302)*(U1(36)*U2(36))
      HU(36) = HESS(303)*(U1(11)*U2(11))+HESS(304)*(U1(13)*U2(36))+HESS(304)*(U1(36)*U2(13))+HESS(305)*(U1(21)*U2(36)) 
     &     +HESS(305)*(U1(36)*U2(21))+HESS(306)*(U1(22)*U2(36))+HESS(306)*(U1(36)*U2(22))+HESS(307)*(U1(26)*U2(35)) 
     &     +HESS(307)*(U1(35)*U2(26))+HESS(308)*(U1(29)*U2(36))+HESS(308)*(U1(36)*U2(29))+HESS(309)*(U1(30)*U2(36)) 
     &     +HESS(309)*(U1(36)*U2(30))+HESS(310)*(U1(31)*U2(36))+HESS(310)*(U1(36)*U2(31))+HESS(311)*(U1(32)*U2(35)) 
     &     +HESS(311)*(U1(35)*U2(32))+HESS(312)*(U1(32)*U2(36))+HESS(312)*(U1(36)*U2(32))+HESS(313)*(U1(33)*U2(35)) 
     &     +HESS(313)*(U1(35)*U2(33))+HESS(314)*(U1(33)*U2(36))+HESS(314)*(U1(36)*U2(33))+HESS(315)*(U1(34)*U2(36)) 
     &     +HESS(315)*(U1(36)*U2(34))+HESS(316)*(U1(35)*U2(36))+HESS(316)*(U1(36)*U2(35))+HESS(317)*(U1(36)*U2(36))
      
      END SUBROUTINE Hess_Vec

C     End of Hess_Vec function
C     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



      END MODULE KPP_Hessian

