      SUBROUTINE STOB (LGRID, LGRID_B, JDATE, JTIME, DT, DX, VEL, ORI) 
      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE CGRID_SPCS            ! CGRID species number and offsets 

      IMPLICIT NONE
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations

      REAL, POINTER      :: LGRID( :,:,:,: )
      REAL, POINTER      :: LGRID_B( :,:,: )
      REAL               :: BCON_OLD(NROWS,NLAYS)
      INTEGER  ::    JDATE      ! current model date, coded YYYYDDD
      INTEGER  ::    JTIME      ! current model time, coded HHMMSS
      INTEGER  ::    DT         
      REAL     ::    DX
      REAL     ::    VEL(:,:,:)
      CHARACTER :: ORI
      CHARACTER( 16 ) :: PNAME = 'STOB'
      CHARACTER( 96 ) :: XMSG = ' '
      INTEGER  ::   C, R, L, S, IOS, IOS1, IOS2
      
      INTEGER, SAVE :: EFX      ! fixed parameter for eastern boundary
      INTEGER, SAVE :: WFX    ! fixed parameter for western boundary
      INTEGER, SAVE :: SFX    ! fixed parameter for southern boundary                             
      INTEGER, SAVE :: NFX    ! fixed parameter for northern boundary   
      INTEGER, SAVE :: FIRSTTIME = .TRUE.

      IF (FIRSTTIME) THEN
         FIRSTTIME = .FALSE.
         
         EFX = MY_NCOLS + 1
         WFX = 2 * MY_NCOLS + MY_NROWS + 4
         SFX = 0
         NFX = MY_NCOLS + MY_NROWS + 3
         
      END IF

C Caculate Sensivity to Boundary conditions      
      IF (ORI .EQ. 'X') THEN     ! xadv to get west and east sensivity
         DO S = 1, NSPCSD
            DO R = 1, NROWS
               DO L = 1, NLAYS
                  ! east
                  IF (VEL(R,L,1) .LT. 0) THEN ! east boundary inflow
                     LGRID_B(EFX+R,L,S) = -1 * VEL(R,L,1) * DT / DX * LGRID(NCOLS,R,L,S) ! EAST
                  ELSE 
                     LGRID_B(EFX+R,L,S) = 0.0 
                  END IF 
                  ! west
                  IF (VEL(R,L,2) .GT. 0) THEN ! west boundary inflow
                     LGRID_B(WFX+R,L,S)  = VEL(R,L,2) * DT / DX * LGRID(1,R,L,S) ! WEST
                  ELSE
                     LGRID_B(WFX+R,L,S)  = 0.0  
                  END IF
               END DO
            END DO
         END DO
      ELSE IF (ORI .EQ. 'Y') THEN  ! yadv to get north and south sensitivity
         DO S = 1, NSPCSD
            DO C = 1, NCOLS
               DO L = 1, NLAYS
                  ! north
                  IF (VEL(C,L,1) .LT. 0) THEN ! north boundary inflow                                                                        
                     LGRID_B(NFX+C,L,S) = -1 * VEL(C,L,1) * DT / DX * LGRID(C,NROWS,L,S) ! NORTH                                                             
                  ELSE
                     LGRID_B(NFX+C,L,S) = 0.0
                  END IF
                  
                  ! south
                  IF (VEL(C,L,2) .GT. 0) THEN ! south boundary inflow                                                                        
                     LGRID_B(SFX+C,L,S)  = VEL(C,L,2) * DT / DX * LGRID(C,1,L,S) ! SOUTH                                                              
                  ELSE
                     LGRID_B(SFX+C,L,S)  = 0.0
                  END IF                  
               END DO           ! L
            END DO              ! C
         END DO                 !S
      END IF
      
      END SUBROUTINE STOB
